
global folder "." // Specify working directory.

cd "$folder\working"
******************************
// faction list from CC
use CC, clear

g temp = floor(job/100)==66
g temp_year=year1 if temp==1
bysort id: egen CYLC=max(temp)
bysort id: egen CYLC_year=min(temp_year)
drop temp*

g temp = floor(job/10)==208
g temp_year=year1 if temp==1
bysort id: egen Shanghai=max(temp)
bysort id: egen Shanghai_year=min(temp_year)
drop temp*

g temp=year1 if job<100 
bysort id: egen CC_year1=min(temp) // the first year enter CC
drop temp
g pre_CC=(year1<CC_year1)
g pre_CC_duration=duration*pre_CC
bysort id: egen sum_pre_CC_duration=sum(pre_CC_duration)

g temp = floor(job/1000)==3 & job>=3011
g duration_military=pre_CC_duration*temp
bysort id: egen sum_pre_CC_duration_military=sum(duration_military) 
g temp_year=year1 if temp==1
bysort id: egen Military=max(temp)
bysort id: egen Military_year=min(temp_year)
g ratio_military=sum_pre_CC_duration_military/sum_pre_CC_duration
replace Military=0 if ratio_military<.5
replace Military_year=. if Military==0
drop temp*
/// revolutionary era
replace Military=1 if red4th==1
replace Military=1 if farmy!=.
replace Military=1 if volunteer==1 | volunteer==2


g Tsinghua=(college==103)
g Tsinghua_year=0

g Princelings=prince
g Princelings_year=0


keep id name name1 *_year CYLC Shanghai Tsinghua Princelings Military 
duplicates drop
replace Shanghai=0 if CYLC==1 // & CYLC_weight>Shanghai_weight // make two faction mutually exclusive
set seed 10
g placebo=(runiform()>.96)
g Neutral=0
replace Neutral=1 if CYLC!=1 & Shanghai!=1 & Princelings!=1 & Military!=1

save factionCC, replace


// faction list from CV
use CV, clear
merge m:1 org using level
drop _m
*tab org if regexm(org,"Shanghai"), sort

g temp=regexm(org,"Shanghai")&(regexm(org,"People's Government") |regexm(org,"CPC,Municipal Committee") | regexm(org,"CPPCC,Municipal Committee") |regexm(org,"Municipal People's Congress") )
*g temp=regexm(org,"Shanghai")
g temp_year=year1 if temp==1

bysort name: egen Shanghai=max(temp)
bysort name: egen Shanghai_year=min(temp_year)
drop temp*

*g temp=(top_org=="CYLC"&level<=0) // only national level CYLC position counts
g temp=(top_org=="CYLC"&level<=1) //  national and provincial level CYLC position counts
*g temp=(top_org=="CYLC"&level<=6)
g temp_year=year1 if temp==1
bysort name: egen CYLC=max(temp)
bysort name: egen CYLC_year=min(temp_year)
drop temp*


g temp=(regexm(org,"Tsinghua"))
g temp_year=year1 if temp==1
bysort name: egen Tsinghua=max(temp)
bysort name: egen Tsinghua_year=min(temp_year)
drop temp*

g temp=year1 if regexm(org,",CPC,Central Committee")
bysort name: egen CC_year1=min(temp) // the first year enter CC
drop temp*
g pre_CC=(year1<CC_year1)
g pre_CC_duration=duration*pre_CC
bysort name: egen sum_pre_CC_duration=sum(pre_CC_duration)

g temp=(regexm(org,",PLA"))
g temp_year=year1 if temp==1
g duration_military=pre_CC_duration*temp
bysort name: egen sum_pre_CC_duration_military=sum(duration_military) 
bysort name: egen Military=max(temp)
bysort name: egen Military_year=min(temp_year)
g ratio_military=sum_pre_CC_duration_military/sum_pre_CC_duration
replace Military=0 if ratio_military<.5
replace Military_year=. if Military==0
drop temp*


g Princelings=prince
replace Princelings=0 if missing(Princelings)
g Princelings_year=0

*replace Shanghai=0 if CYLC==1 // & CYLC_weight>Shanghai_weight // make two faction mutually exclusive
*replace Shanghai=0 if CYLC_year>Shanghai_year // make two faction mutually exclusive

keep name  chinese_name CYLC* Shanghai* Tsinghua* Military* Princelings*
duplicates drop
set seed 10
g placebo=(runiform()>.96)
g placebo_year=0
g Neutral=0
replace Neutral=1 if CYLC!=1 & Shanghai!=1 & Princelings!=1 & Military!=1

save factionCV1, replace


// CV centric
use factionCV1, clear
foreach var of varlist CYLC* Shanghai* Tsinghua* Military* Princelings* {
rename `var' `var'1
}
merge m:1 name using link_CC_CV
keep if _m==3
drop _m
merge 1:1 id  using factionCC
keep if _m==3
drop _m
mdesc id
isid id 
local factionlist CYLC Shanghai Tsinghua Military Princelings
foreach faction of local factionlist {
replace `faction'=(`faction'1==1|`faction'==1)
}

local factionlist CYLC Shanghai Tsinghua Military Princelings
foreach faction of local factionlist {
replace `faction'_year=min(`faction'_year1,`faction'_year)
}
drop  *1 
replace Shanghai=0 if CYLC_year<Shanghai_year // take early experience as the faction affilication
replace CYLC=0 if CYLC_year>=Shanghai_year // take early experience as the faction affilication
save factionCV_centric1, replace


use factionCV1, clear
merge m:1 name using link_CC_CV
keep if _m==1
drop _m

append using factionCV_centric1
replace Shanghai=0 if CYLC==1 // & CYLC_weight>Shanghai_weight // make too faction mutually exclusive

capture: drop placebo placebo_year
g placebo=(runiform()>.96)
g placebo_year=0
capture: drop Neutral
g Neutral=0
replace Neutral=1 if CYLC!=1 & Shanghai!=1 & Princelings!=1 & Military!=1

save factionCV_centric1, replace

* levels at CC18
use promotion_CC, clear
keep if CC_no==18
rename name chinese_name
keep name1 chinese_name ladder
duplicates drop 
save CC18, replace

*******************************************************
* CC 19 data
set more off
import excel using "$folder\CC Data\CC19.xlsx", sheet(CC) firstrow clear
drop if substr(Name,1,1)=="("
rename Name name1
rename Hanzi chinese_name 
rename CIMCCDI Position
g PB=(strpos(Position, "PB")>0)
g PBSC=(strpos(Position, "PSC")>0)
g Secretariat=(strpos(Position, "ST")>0)
g CMC=(strpos(Position, "CMC")>0)
g CCDI=(strpos(Position, "CCDI")>0)
g SCCCDI=(strpos(Position, "SC-CCDI")>0)
g MT=(K=="§")
g Female=(K=="♀")
g AC=0

keep chinese_name name1 PB PBSC Secretariat CMC CCDI SCCCDI MT Female Ethnicity
duplicates report name
save CC19_1, replace

set more off
import excel using "$folder\CC Data\AC19.xlsx", firstrow clear
drop if substr(Name,1,1)=="("
rename Name name1
rename Hanzi chinese_name 
g AC=1
g MT=(K=="§")
g Female=(K=="♀")
drop Office Rank K
append using CC19_1
foreach var of varlist AC-SCCCDI {
replace `var'=0 if `var'==.
}
g uid=_n
save CC19, replace

  
  
  

use factionCV_centric1, clear
capture: drop id
g id=_n
save temp, replace
matchit id chinese_name using CC19.dta , idusing(uid) txtusing(chinese_name) 
keep if similscore>=1
duplicates report chinese_name
duplicates drop uid, force
keep id uid
save CC19_faction_link, replace

use CC19, clear
*keep if PB==1
merge 1:1 uid using CC19_faction_link
keep if _m==3
drop _m
save CC19_temp1, replace



set more off
import excel using "$folder\CC Data\19thCC_BlindaChen.xlsx", firstrow clear



keep if PB==1

merge 1:1 id using temp
keep if _m==3
drop _m

merge 1:1 chinese_name using CC18
drop if _m==2
drop _m

replace ladder=5 if ladder==.
g prom=0
replace prom=1 if ladder>=3 
replace prom=1 if ladder>=2 & PBSC==1

keep name1 chinese_name CYLC Shanghai Military Princelings  PBSC PB ladder prom
gsort - PBSC
g Neutral=(CYLC==0&Shanghai==0)



replace Neutral=0 if name1=="Xi Jinping"
replace Shanghai=1 if name1=="Xi Jinping"


tabstat  Shanghai Neutral CYLC if PBSC==1&prom==1,  s( mean ) 
tabstat  Shanghai Neutral CYLC if PB==1&prom==1,  s( mean ) 


replace Neutral=1 if name1=="Xi Jinping"
replace Shanghai=0 if name1=="Xi Jinping"

tabstat  Shanghai Neutral CYLC if PBSC==1&prom==1,  s( mean ) 
tabstat  Shanghai Neutral CYLC if PB==1&prom==1,  s( mean ) 








set more off
import excel using "$folder\CC Data\19thCC_BlindaChen.xlsx", firstrow clear
append using CC19_1
foreach var of varlist AC-SCCCDI {
replace `var'=0 if `var'==.
}
save CC19, replace

















