*************
* Hand collected CV
cd ".\working"
import excel using ".\data\Corruption.xlsx", clear sheet(hand) first
drop if missing(j1)
reshape long j s e, i(chinese_name) j(k)
drop if missing(j)
save temp, replace


use temp, clear
split birthdayym, p("-")
destring birthdayym1, g(birth)
g gender_dum=(gender=="M")
g ethnic_dum=(nationality!="")
g CYLC=regexm(j,"共青团")
g Shanghai=0
replace Shanghai=1 if chinese_name=="周波"
g Military=(regexm(j,"军")|regexm(j,"兵"))
g Tsinghua=regexm(j,"清华")

foreach var of varlist CYLC Shanghai Military Tsinghua {
bysort chinese_name: egen `var'_temp=max(`var')
drop `var'
rename  `var'_temp `var'
}

keep chinese_name  birth gender_dum ethnic_dum CYLC Shanghai Military Tsinghua
duplicates drop 
save corruption_hand_collected, replace


****************
* Hand collected sample
cd ".\working"
import excel using ".\data\Corruption.xlsx", clear sheet(hand) first
drop if missing(j1)
reshape long j s e, i(chinese_name) j(k)
drop if missing(j)
save temp, replace

use temp, clear
split birthdayym, p("-")
destring birthdayym1, g(birth)
g gender_dum=(gender=="M")
g ethnic_dum=(nationality!="")
*g CYLC=regexm(j,"共青团")
*g CYLC=regexm(j,"共青团中央")

g CYLC=(regexm(j,"共青团")&regexm(j,"省委"))
replace CYLC=1 if (j=="共青团黑龙江省第八届委员会副书记、党组副书记" | j=="共青团黑龙江省第九届委员会书记、党组书记" )


g Shanghai=0
replace Shanghai=1 if chinese_name=="周波"
g Military=(regexm(j,"军")|regexm(j,"兵"))
g Tsinghua=regexm(j,"清华")
g Princelings=(prince=="1")
foreach var of varlist CYLC Shanghai Military Tsinghua {
bysort chinese_name: egen `var'_temp=max(`var')
drop `var'
rename  `var'_temp `var'
}

keep chinese_name  birth gender_dum ethnic_dum CYLC Shanghai Princelings Military  Tsinghua
duplicates drop 
g corruption=1
save corruption_HC, replace

****************
* CV sample
cd ".\working"
import excel using ".\data\Corruption.xlsx", clear sheet(All) first
rename name chinese_name
rename name1 name
keep chinese_name
save corruption_list, replace

use CV, clear
merge m:1 name using factionCV_centric
drop _m
merge m:1  chinese_name using corruption_list
drop if _m==2
g corruption=(_m==3)
drop _m
keep if year2>=2007
duplicates drop chinese_name, force
g ethnic_dum=(nationality!="Han" & nationality!="NA")
g gender_dum=(gender=="male" | gender=="NA")
destring byear, replace force
g birth=byear
keep chinese_name id corruption Shanghai CYLC Tsinghua Military Princelings birth ethnic_dum gender_dum
g sample="CV"
save corruption_CV, replace

****************
* CC sample
use CC, clear
merge m:1 id using factionCC_centric
keep if _m==3
drop _m
rename name chinese_name
merge m:1  chinese_name using corruption_list
g corruption=(_m==3)
drop if _m==2
drop _m

drop if death<=2012
keep if year2>=2007

g ladder=5
g CC=0
g AC=0
replace ladder=1 if (job==6001 | job==6002) // PBSC
replace ladder=2 if (job>=6003 & job<=6004) // PB

foreach num of numlist 8/18 {
replace ladder=3 if job==`num' //Full member of the Central Committee
replace ladder=4 if job==`num'+.1 //alternate member of the Central Committee
replace CC=1 if job==`num'
replace AC=1 if job==`num'+.1
}

g PBSC=(job==6001 | job==6002) 
g PB=(job>=6003 & job<=6004)
foreach var of varlist PBSC PB CC AC {
g temp=`var'
drop `var'
bysort id: egen `var'=max(temp)
drop temp
}
bysort id: egen min_ladder=min(ladder)
keep if ladder==min_ladder
duplicates drop id, force

g ethnic_dum=(ethnic>0)
g abroad_dum=(abroad>0)
g gender_dum=gender

keep  id name chinese_name corruption Shanghai CYLC Tsinghua Military Princelings birth ethnic_dum gender_dum min_ladder  AC CC PB PBSC
g sample="CC"
save corruption_CC, replace

*******************
* Combine three sournces of faction: CV, CC, HandCollected 
use corruption_CC, clear
g CC_sample=1
append using corruption_CV
bysort id: g N=_N
tab N
drop if N==2 & sample=="CV"
drop N

bysort chinese_name: g N=_N
tab N
drop if N==2 & sample=="CV"

append using corruption_HC

foreach var of varlist AC CC PB PBSC {
replace `var'=0 if missing(`var')
}

foreach var of varlist birth {
egen median_`var'=median(`var')
replace `var'=median_`var' if `var'==.
drop median_`var'
}
g age=2016-birth 

replace min_ladder=5 if missing(min_ladder)
save corruption_All, replace


use corruption_All, clear
lab var gender_dum "Gender"
lab var ethnic_dum "Ethnic Minority"
lab var age "Age"
lab var age "Age Square"
lab var age "Age Cube"

egen agez=std(age)
g agez2=agez^2
g agez3=agez^3


g Military_Only=(Military==1)
replace Military_Only=0 if CYLC+Shanghai+Princelings>0

g CCAC=(CC==1|AC==1)

eststo clear
eststo Inc_Military1: reg corruption  CYLC Shanghai   Princelings  Military  , robust
test CYLC=Shanghai
estadd scalar  pvalue=`r(p)'
estadd local indi "No"
estadd local level "No"
eststo Inc_Military2: reg corruption  CYLC Shanghai   Princelings  Military gender_dum ethnic_dum agez agez2 agez3  , robust
test CYLC=Shanghai
estadd scalar  pvalue=`r(p)'
estadd local indi "Yes"
estadd local level "No"
eststo Inc_Military3: reg corruption  CYLC Shanghai   Princelings  Military gender_dum ethnic_dum agez agez2 agez3  AC CC PB PBSC , robust
test CYLC=Shanghai
estadd scalar  pvalue=`r(p)'
estadd local indi "Yes"
estadd local level "Yes"
esttab using corruption.csv,  se star(* 0.1 ** 0.05 *** 0.01) ar2  br lab mtitle  replace
esttab using "D:\Dropbox\Chinese Politics\paper\table_figure\corruption.tex", ///
s( pvalue indi level N r2_a, label( "p-value (CYLC=Shanghai)"  "Individual Attributes" "Level F.E." "Observations" "Adj. R-squared") ) ///
 mtitles("Corruption" "Corruption" "Corruption" "Corruption" )  ///
 keep (CYLC Shanghai  Princelings  Military) ///
width(\hsize) se star(* 0.1 ** 0.05 *** 0.01) ar2  br lab mtitle  replace nocon nonotes




tab  corruption CYLC
tab corruption Shanghai
tab corruption Princelings
tab corruption Military

/*
******************************
* Sample: CC only
cd "C:\Users\xiao\Dropbox\Chinese Politics\working"
import excel using "C:\Users\xiao\Dropbox\Chinese Politics\data\Corruption.xlsx", clear sheet(All) first
rename name chinese_name
rename name1 name
keep chinese_name
save corruption_list, replace


use CC, clear
merge m:1 id using factionCC_centric
drop _m
rename name chinese_name
merge m:1  chinese_name using corruption_list
g corruption=(_m==3)
drop _m
*drop if name1=="Liu Zhijun"
g ladder=5
g CC=0
g AC=0
replace ladder=1 if (job==6001 | job==6002) // PBSC
replace ladder=2 if (job>=6003 & job<=6004) // PB
set more off
foreach num of numlist 8/18 {
replace ladder=3 if job==`num' //Full member of the Central Committee
replace ladder=4 if job==`num'+.1 //alternate member of the Central Committee
replace CC=1 if job==`num'
replace AC=1 if job==`num'+.1
}

g PBSC=(job==6001 | job==6002) 
g PB=(job>=6003 & job<=6004)

foreach var of varlist PBSC PB CC AC {
g temp=`var'
drop `var'
bysort id: egen `var'=max(temp)
drop temp
}

g dummy=(job>=17&job<19)
bysort id: egen max_dummy=max(dummy)
keep if max_dummy==1

drop if death<=2012
bysort id: egen min_ladder=min(ladder)
keep if ladder==min_ladder
duplicates drop id, force

g ethnic_dum=(ethnic>0)
g abroad_dum=(abroad>0)
g gender_dum=gender

eststo clear
eststo All: reg corruption  CYLC Shanghai Military  Princelings  gender ethnic_dum edu abroad_dum mishu birth PBSC PB CC , robust
test CYLC=Shanghai
esttab using corruption.csv,  se star(* 0.1 ** 0.05 *** 0.01) ar2  br lab mtitle  replace

tab  corruption CYLC
tab corruption Shanghai
tab corruption Princelings
tab corruption Military
