

cd ".\working"

use CC_8_18_nogap, clear
replace CC=1 if PB==1 // change CC definition to include PC (which already include SC)
sort id year AC CC PB PBSC
duplicates drop id year  AC CC PB PBSC, force // 

gen faction="Neutral" // make groups mutually exclusive
replace faction="Military" if Military==1
replace faction="CYLC" if CYLC==1
replace faction="Shanghai" if Shanghai==1
replace faction="Princelings" if Princelings==1

collapse (sum) Score AC CC PB PBSC, by(faction year)

foreach var of varlist Score AC CC PB PBSC {
bysort year: egen sum_`var'=sum(`var')
g `var'_share_ = `var'/sum_`var'
}
keep faction year *_share_
reshape wide *_share_, i(year) j(faction) string

tsset year
save faction_share_by_level, replace 

* AC/CC ratio as malapportionment
use faction_share_by_level, clear
foreach	faction in "CYLC" "Shanghai" "Princelings" "Military" "Neutral" {
foreach level in "CC"  {
	g d_`faction'_`level'=abs(`level'_share_`faction' - AC_share_`faction')/2
	}
	
}


egen malapportionment=rowtotal(d_*)
replace malapportionment=malapportionment/2
tsline malapportionment
keep malapportionment year
save malapportionment, replace


* 5-year change stability 
use faction_share_by_level, clear
foreach	faction in "CYLC" "Shanghai" "Princelings" "Military" "Neutral" {
	foreach level in  "CC"   {
	sort year
	g d_`faction'_`level'=abs(`level'_share_`faction'-`level'_share_`faction'[_n-5])
	}
}


egen instability=rowtotal(d_*)
replace instability=instability/2
tsline instability
keep instability year
save instability, replace






capture: erase __000000.txt
freduse RGDPNACNA666NRUG MKTGDPCNA646NWDB, clear
g year = year(daten)
tsset year
rename RGDPNACNA666NRUG GDP //real gdp
rename MKTGDPCNA646NWDB GDP2 // nominal gdp
g GDP_gr = (GDP/L.GDP-1)*100
g GDP_gr2 = (GDP2/L.GDP2-1)*100


merge 1:1 year using  "malapportionment.dta"
keep if _m==3
drop _m
merge 1:1 year using  "instability.dta"
keep if _m==3
drop _m


g period = 1 if (year>=1954 & year<=1976)
replace period = 2 if year>1976 & year<=1988
replace period = 3 if year>1988

foreach var of varlist GDP_gr instability malapportionment {
bysort period: egen mean_`var'= mean(`var')

}

twoway (tsline GDP_gr mean_GDP_gr, lpattern(solid dash) lcolor(black)),  ///
 title("")  graphregion(color(white)) bgcolor(white) ylabel(, nogrid valuelabels ) ///
 xlabel(1955(10)2015, valuelabels angle(0)) xtitle("Year") ///
 ytitle("GDP growth") title("") legend(off) xline(1977) xline(1989)
graph export "$folder\paper\table_figure\gdp_growth.pdf", replace


twoway (tsline instability , lpattern(solid ) lcolor(black)),  ///
 title("")  graphregion(color(white)) bgcolor(white) ylabel(, nogrid valuelabels ) ///
 xlabel(1955(10)2015, valuelabels angle(0)) xtitle("Year") ///
 ytitle("Instability") title("") legend(off)  xline(1977)
graph export "$folder\paper\table_figure\instability.pdf", replace

twoway (tsline malapportionment, lcolor(black)) ,  ///
 title("")  graphregion(color(white)) bgcolor(white) ylabel(, nogrid valuelabels ) ///
 xlabel(1955(10)2015, valuelabels angle(0)) xtitle("Year") ///
 ytitle("Malapportionment") title("") legend(off) xline(1977)  
graph export "$folder\paper\table_figure\malapportionment.pdf", replace

tsset year
foreach var of varlist GDP malapportionment instability   {
g ln_`var' = log(`var')
g dln_`var'=D.ln_`var'
}




var dln_GDP dln_malapportionment dln_instability, lags(1) dfk 
irf create irf1, set(irf1, replace)
irf graph oirf, impulse(dln_malapportionment dln_instability) response(dln_GDP ) 

use "irf1.irf",clear
keep if impulse == "dln_malapportionment" & response == "dln_GDP"
keep step oirf stdoirf
gen upper_ci1= oirf +1.645*stdoirf
gen lower_ci1 = oirf - 1.645*stdoirf


twoway line oirf step , lc(black)  ///
    || line upper_ci1 step,lc(gray)  lpattern(dot) /// 
	|| line lower_ci1 step,lc(gray)  lpattern(dot) /// 
    || ,  ///
 title("")  graphregion(color(white)) bgcolor(white) ylabel(-.015(.005)0, nogrid valuelabels ) ///
 xlabel(, valuelabels angle(0)) xtitle("Year") ///
 ytitle("GDP growth") title("") legend(off) yline(0,lpattern("-") lcolor(black))  
graph export "$folder\paper\table_figure\irf_gdp_malapportionment.pdf", replace

	
	
use "irf1.irf",clear
keep if impulse == "dln_instability" & response == "dln_GDP"
keep step oirf stdoirf
gen upper_ci1= oirf +1.645*stdoirf
gen lower_ci1 = oirf - 1.645*stdoirf
twoway line oirf step , lc(black)  ///
    || line upper_ci1 step,lc(gray)  lpattern(dot) /// 
	|| line lower_ci1 step,lc(gray)  lpattern(dot) /// 
    || ,  ///
 title("")  graphregion(color(white)) bgcolor(white) ylabel(-.015(.005)0, nogrid valuelabels ) ///
 xlabel(, valuelabels angle(0) ) xtitle("Year") /// 
 ytitle("GDP growth") title("") legend(off) yline(0,lpattern("-") lcolor(black))  
graph export "$folder\paper\table_figure\irf_gdp_instability.pdf", replace

