cd ./working

// CC data
use CC, clear
g ladder=5
replace ladder=1 if (job==6001 | job==6002) // PBSC
replace ladder=2 if (job>=6003 & job<=6004) // PB

g CC_no=.
foreach num of numlist 8/18 {
replace ladder=3 if job==`num' //Full member of the Central Committee
replace ladder=4 if job==`num'+.1 //alternate member of the Central Committee
replace CC_no=`num' if (job==`num' | job==`num'+.1)
}

g CC=0
g AC=0
foreach num of numlist 8/18 {
replace AC=1 if job==`num'+.1
replace CC=1 if job==`num' //Full member of the Central Committee
}
g PB=(job>=6001 & job<=6004) // PB
g PBSC=(job>=6001 & job<=6002) //PBSC

save promotion_raw, replace

local yearlist 1956 1969 1973 1977 1982 1987 1992 1997 2002 2007 2012
foreach year of local yearlist   {
use promotion_raw, clear
keep if  year1<=`year' & year2>`year' //  restrict to new CC
g year=`year'
save promotion_`year', replace
}

clear
local yearlist 1956 1969 1973 1977 1982 1987 1992 1997 2002 2007 2012
foreach year of local yearlist   {
append using promotion_`year'
}

// general secretary's faction
merge m:1 year using faction_secretary_ccyear
drop _m

bysort id year: egen min_CC_no=min(CC_no)
replace CC_no=min_CC_no
drop min_CC_no
keep if CC_no!=. // keep only if in AC/CC
bysort id year: egen min_ladder=min(ladder)
keep if ladder==min_ladder
drop min_ladder

duplicates drop id year, force
sort id year 
g age=year-birth

bysort id: g promotion=-(ladder[_n+1]-ladder[_n])  // promotion in the future
*bysort id: replace promotion=. if CC_no[_n]!=CC_no[_n+1]-1  // drop cases with gap, not frequent
replace promotion=. if year==2012 // 18th CC, end of the sample
bysort id: replace promotion=0 if (_n==_N&year!=2012) // retirement is no promotion
g dum_promotion=(promotion>0)
replace dum_promotion=. if promotion==.
replace dum_promotion=. if year==2012
sort id year
bysort id: g retire=(_n==_N)
replace retire=. if year==2012
g retire_ab=retire
replace retire_ab=0 if ladder>=3 & age>=64-5 & death>year+5 // five years later, older than 64
replace retire_ab=0 if ladder<=2 & age>=68-5 & death>year+5 // five years later, older than 64
bysort id: g firstCCAC=(_n==1&year!=1956) // first time in CC/AC
g CCAC=1

sort PB id year
bysort PB id: g firstPB=(_n==1&year!=1956&PB==1)

sort PBSC id year
bysort PBSC id: g firstSC=(_n==1&year!=1956&PBSC==1)

g age59_62=(age>=64-5 & age<68-5)
g age63=(age>=68-5)
egen agez=std(age)
g agez2=agez^2
g agez3=agez^3

*merge m:1 id using factionCC_centric
merge m:1 id using factionCC

keep if _m==3
drop _m

g college_dum=(college!=.)
g gschool_dum=(gschool!=.)
g mishu_dum=(mishu>0)
g ethnic_dum=(ethnic>0)
g abroad_dum=(abroad>0)

* current and previous
foreach var of varlist CYLC Shanghai Tsinghua Military Princelings {
g `var'_secretary_L01=(`var'_secretary+`var'_secretary_L1>0)
}
* interaction (promotion is in F1)
foreach var of varlist CYLC Shanghai Tsinghua Military Princelings {
g `var'_`var'_sec=`var'*(`var'_secretary_F1)
}
/*
foreach var of varlist CYLC Shanghai Tsinghua Military Princelings {
g `var'_`var'_sec=`var'*(`var'_secretary+`var'_secretary_F1>0)
}
*/

save promotion_CC, replace

**********************************
* above_same_below ratio
use promotion_CC, clear
g one=1
collapse (sum) tot=one, by(CC_no ladder)
save tot, replace


use promotion_CC, clear
g f = 0
replace f = 1 if CYLC==1
replace f = 2 if Shanghai==1
g one=1
*replace one=0 if age>64-5 & ladder>=3
*replace one=0 if age>68-5 & ladder<=2
collapse (sum) num_same=one, by(CC_no f ladder)
egen group=group(f ladder)
tsset group CC_no
tsfill, full
bysort group: carryforward f ladder, replace
gsort group -CC_no
bysort group: carryforward f ladder, replace
replace num_same=0 if num_same==.

merge m:1 CC_no ladder using tot
g share=num_same/tot
keep CC_no ladder f share
sort CC_no ladder f
save share, replace

use share, clear
replace ladder=ladder+1
rename share share_L1
save share_L1, replace

use share, clear
replace ladder=ladder+2
rename share share_L2
save share_L2, replace

use share, clear
replace ladder=ladder-1
rename share share_H1
save share_H1, replace

use share, clear
replace ladder=ladder-2
rename share share_H2
save share_H2, replace

use share, clear
merge 1:1 CC_no ladder f using share_L1
drop if _m==2
drop _m
merge 1:1 CC_no ladder f using share_L2
drop if _m==2
drop _m
merge 1:1 CC_no ladder f using share_H1
drop if _m==2
drop _m
merge 1:1 CC_no ladder f using share_H2
drop if _m==2
drop _m
save share_above_below, replace

**********************************
* competition
use promotion_CC, clear
g candidate=1 
replace candidate=0 if age>64-5 & ladder>=3
replace candidate=0 if age>68-5 & ladder<=2
collapse (sum) tocandidate=candidate , by(CC_no ladder)
save tocandidate, replace

use promotion_CC, clear
g candidate=1 
replace candidate=0 if age>64-5 & ladder>=3
replace candidate=0 if age>68-5 & ladder<=2
g f = 0
replace f = 1 if CYLC==1
replace f = 2 if Shanghai==1
collapse (sum) cocandidate=candidate , by(CC_no f ladder)
save cocandidate, replace

use promotion_CC, clear
g opening=1 
replace opening=0 if age<=64-5 & ladder>=3
replace opening=0 if age<=68-5 & ladder<=2
g f = 0
replace f = 1 if CYLC==1
replace f = 2 if Shanghai==1
replace ladder=ladder+1
collapse (sum) opening , by(CC_no f ladder)
save opening, replace


**************************************
* power score share
use CC_8_18_nogap, clear
g f = 0
replace f = 1 if CYLC==1
replace f = 2 if Shanghai==1

g Seat=CC+AC
collapse (sum) Score Seat, by(f year)
g CC_no=8 if year==1956
replace CC_no=9 if year==1969
replace CC_no=10 if year==1973
replace CC_no=11 if year==1977
replace CC_no=12 if year==1982
replace CC_no=13 if year==1987
replace CC_no=14 if year==1992
replace CC_no=15 if year==1997
replace CC_no=16 if year==2002
replace CC_no=17 if year==2007
replace CC_no=18 if year==2012
keep if CC_no!=.
bysort year: egen sum_score=sum(Score)
bysort year: egen sum_seat=sum(Seat)

g share_score=Score/sum_score
g share_seat=Seat/sum_seat
*keep CC_no f share*
save overall_share, replace

* merge
use promotion_CC, clear
g f = 0
replace f = 1 if CYLC==1
replace f = 2 if Shanghai==1

merge m:1  CC_no ladder f using above_same_below
drop _m
merge m:1  CC_no  f using overall_share
drop _m
merge m:1  CC_no ladder f using opening
drop if _m==2
drop _m
merge m:1  CC_no ladder f using cocandidate
drop _m
merge m:1  CC_no ladder using tocandidate
drop _m
merge m:1  CC_no ladder f using share_above_below
drop if _m==2
drop _m
// counterfactual score if promoted
foreach var of varlist Score  {
g `var'1=`var'
replace `var'1=`var'+1 if ladder==4
replace `var'1=`var'+2 if ladder==3
replace `var'1=`var'+1 if ladder==2
replace `var'1=`var' if ladder==1
}
g share_score1=Score1/sum_score

// counterfactual seat if promoted
foreach var of varlist Seat  {
g `var'1=`var'
replace `var'1=`var'+1 
}
g share_seat1=Seat1/sum_seat
g leaderdummy=CYLC_CYLC_sec+Shanghai_Shanghai_sec 
g leaderdummy2=CYLC*CYLC_secretary+Shanghai*Shanghai_secretary
g otcan_share= (tocandidate-cocandidate)/tot_same
g cocan_share= (cocandidate)/tot_same
save promotion_CC_above_same_below, replace


* Summary Stats
use promotion_CC, clear
rename gender Gender
rename age Age
rename college_dum College
rename gschool_dum GradSchool
rename abroad_dum Abroad
rename mishu_dum Mishu
rename ethnic_dum EthnicMinor
rename dum_promotion Promotion
rename retire Retirement

replace Promotion=0 if Promotion==.
replace Retirement=0 if Retirement==.

estpost tabstat Gender Age College GradSchool Abroad Mishu ///
EthnicMinor Promotion Retirement CYLC Shanghai Military Princelings, by(CC_no) ///
statistics(mean sd) columns(statistics) listwise

esttab . using "D:\Dropbox\Chinese Politics\paper\table_figure\SummaryCC.tex", ///
main(mean %9.2f) aux(sd %9.2f) nostar unstack  noobs nonote nomtitle nonumber   nogaps   replace  



* Summary Stats by ladder
use promotion_CC, clear
isid id CC_no
rename gender Male
rename age Age
rename college_dum College
rename gschool_dum GradSchool
rename abroad_dum Abroad
rename mishu_dum Mishu
rename ethnic_dum EthnicMinor
rename dum_promotion Promotion
rename retire Retirement

replace Promotion=0 if Promotion==.
replace Retirement=0 if Retirement==.
keep if  CC_no>=11

foreach var of varlist rank12-rank18 {
replace ladder = 0 if ladder==1 & `var'<=.2
}
replace ladder = 0 if CC_no==11 & (name=="华国锋"|name=="邓小平")

g ladder_string = "Top Leader"
replace ladder_string = "PBSC" if ladder==1
replace ladder_string = "PB" if ladder==2
replace ladder_string = "CC" if ladder==3
replace ladder_string = "AC" if ladder==4


estpost tabstat  CYLC Shanghai Military Princelings Male Age College GradSchool Abroad Mishu ///
EthnicMinor  , by(ladder) ///
statistics( mean sd) columns(statistics) listwise nototal

esttab . using "D:\Dropbox\Chinese Politics\paper\table_figure\summary_stats_by_ladder_no_label.tex", ///
main(mean %9.3f) aux(sd %9.3f) nostar unstack  noobs nonote nomtitle nonumber   nogaps  width(\hsize)  replace  

bb
***********
* Graph
* scatter age year in PB
use promotion_CC, clear
g Unclassified=(CYLC+Shanghai+Princelings+Military==0)

local factionlist CYLC 
foreach faction of local factionlist {
#delimit ;
twoway 
(scatter age year if ladder<=2 & `faction'== 1, msize(small) mcolor(maroon))
, 
yline(68, lpattern(dash))
xline(1990)
title("`faction'")
xtitle("")
ytitle("")
ylab(, nogrid)
xscale(range(1952 2014) noextend)
xlabel(1952(5)2014,angle(45)) 

graphregion(color(white))
;
#delimit cr
graph save age_`faction', replace
}

local factionlist  Shanghai 
foreach faction of local factionlist {
#delimit ;
twoway 
(scatter age year if ladder<=2 & `faction'== 1, msize(small) mcolor(navy))
, 
yline(68, lpattern(dash))
xline(1990)
title("`faction'")
xtitle("")
ytitle("")
ylab(, nogrid)
xscale(range(1952 2014) noextend)
xlabel(1952(5)2014,angle(45)) 

graphregion(color(white))
;
#delimit cr
graph save age_`faction', replace
}

local factionlist  Military 
foreach faction of local factionlist {
#delimit ;
twoway 
(scatter age year if ladder<=2 & `faction'== 1, msize(small) mcolor(forest_green))
, 
yline(68, lpattern(dash))
xline(1990)
title("`faction'")
xtitle("")
ytitle("")
ylab(, nogrid)
xscale(range(1952 2014) noextend)
xlabel(1952(5)2014,angle(45)) 

graphregion(color(white))
;
#delimit cr
graph save age_`faction', replace
}

local factionlist  Princelings 
foreach faction of local factionlist {
#delimit ;
twoway 
(scatter age year if ladder<=2 & `faction'== 1, msize(small) mcolor(dkorange))
, 
yline(68, lpattern(dash))
xline(1990)
title("`faction'")
xtitle("")
ytitle("")
ylab(, nogrid)
xscale(range(1952 2014) noextend)
xlabel(1952(5)2014,angle(45)) 

graphregion(color(white))
;
#delimit cr
graph save age_`faction', replace
}

graph combine  age_CYLC.gph age_Shanghai.gph age_Military.gph age_Princelings.gph ,  ycommon  col(2)  graphregion(color(white))
graph export  "D:\Dropbox\Chinese Politics\paper\table_figure\age_scatter_PB.pdf", replace 


* scatter age year in CC
use promotion_CC, clear
g Unclassified=(CYLC+Shanghai+Princelings+Military==0)
keep if firstCCAC==1
local factionlist CYLC 
foreach faction of local factionlist {
#delimit ;
twoway 
(scatter age year if ladder>=3 & `faction'== 1, msize(small) mcolor(maroon))
, 
yline(64, lpattern(dash))
xline(1990)
title("`faction'")
xtitle("")
ytitle("")
ylab(, nogrid)
xscale(range(1952 2014) noextend)
xlabel(1952(5)2014,angle(45)) 

graphregion(color(white))
;
#delimit cr
graph save age_`faction', replace
}

local factionlist  Shanghai 
foreach faction of local factionlist {
#delimit ;
twoway 
(scatter age year if ladder>=3 & `faction'== 1, msize(small) mcolor(navy))
, 
yline(64, lpattern(dash))
xline(1990)
title("`faction'")
xtitle("")
ytitle("")
ylab(, nogrid)
xscale(range(1952 2014) noextend)
xlabel(1952(5)2014,angle(45)) 

graphregion(color(white))
;
#delimit cr
graph save age_`faction', replace
}

local factionlist  Military 
foreach faction of local factionlist {
#delimit ;
twoway 
(scatter age year if ladder>=3 & `faction'== 1, msize(small) mcolor(forest_green))
, 
yline(64, lpattern(dash))
xline(1990)
title("`faction'")
xtitle("")
ytitle("")
ylab(, nogrid)
xscale(range(1952 2014) noextend)
xlabel(1952(5)2014,angle(45)) 

graphregion(color(white))
;
#delimit cr
graph save age_`faction', replace
}

local factionlist  Princelings 
foreach faction of local factionlist {
#delimit ;
twoway 
(scatter age year if ladder>=3 & `faction'== 1, msize(small) mcolor(dkorange))
, 
yline(64, lpattern(dash))
xline(1990)
title("`faction'")
xtitle("")
ytitle("")
ylab(, nogrid)
xscale(range(1952 2014) noextend)
xlabel(1952(5)2014,angle(45)) 

graphregion(color(white))
;
#delimit cr
graph save age_`faction', replace
}

graph combine  age_CYLC.gph age_Shanghai.gph age_Military.gph age_Princelings.gph ,  ycommon  col(2)  graphregion(color(white))
graph export  "D:\Dropbox\Chinese Politics\paper\table_figure\age_scatter_CC.pdf", replace


********************************************
* graph: promotion premium and faction

use promotion_CC, clear
statsby _b _se, by(year_sec): reg dum_promotion CYLC Shanghai Princelings Military ///
 age59 age63 gender college_dum gschool_dum mishu_dum ethnic_dum abroad_dum, robust
local factionlist  CYLC Shanghai Princelings Military
foreach faction of local factionlist {
g hi_`faction' = _b_`faction' + 1.96*_se_`faction'
g lo_`faction' = _b_`faction' - 1.96*_se_`faction'
}
tsset year_sec
save promotion_graph, replace



use promotion_graph, clear
// CYLC
local factionlist  CYLC 
foreach faction of local factionlist {
quietly: sum _b_`faction'
local ymin 1
local ymax -.5

#delimit ;
twoway 
(scatteri `ymin' `=1981' `ymin'  `=1986', bcolor(gs15) recast(area) )
(scatteri `ymax' `=1981' `ymax'  `=1986', bcolor(gs15) recast(area) )
(scatteri `ymin' `=2002' `ymin'  `=2011', bcolor(gs15) recast(area) )
(scatteri `ymax' `=2002' `ymax'  `=2011', bcolor(gs15) recast(area) )
(tsline  _b_`faction', lpattern(solid) lcolor(maroon))
(rcap hi_`faction' lo_`faction' year_sec, lcolor(gs6))
	   ,
title(`faction')
xtitle("")
xscale(range(1967 2014) noextend)
xlabel(1967(5)2014,angle(45)) 
ylab(, nogrid)
graphregion(color(white))
legend(   off )
;
#delimit cr
graph save promotion_`faction', replace
}

// Shanghai

local factionlist  Shanghai 
foreach faction of local factionlist {
quietly: sum _b_`faction'
local ymin .6
local ymax -.4
#delimit ;
twoway 
(scatteri `ymin' `=1989' `ymin'  `=2001', bcolor(gs15) recast(area) )
(scatteri `ymax' `=1989' `ymax'  `=2001', bcolor(gs15) recast(area) )
(scatteri `ymin' `=2012' `ymin'  `=2014', bcolor(gs15) recast(area) )
(scatteri `ymax' `=2012' `ymax'  `=2014', bcolor(gs15) recast(area) )
(tsline  _b_`faction', lpattern(solid) lcolor(navy))
(rcap hi_`faction' lo_`faction' year_sec, lcolor(gs6))
 ,
title(`faction')
xtitle("")
xscale(range(1967 2014) noextend)
xlabel(1967(5)2014,angle(45)) 
ylab(, nogrid)
graphregion(color(white))
legend(   off )
;
#delimit cr
graph save promotion_`faction', replace
}

// Military
local factionlist  Military 
foreach faction of local factionlist {
quietly: sum _b_`faction'
local ymin .2
local ymax -.2
#delimit ;
twoway 
(scatteri `ymin' `=1969' `ymin'  `=1988', bcolor(gs15) recast(area) )
(scatteri `ymax' `=1969' `ymax'  `=1988', bcolor(gs15) recast(area) )
(tsline  _b_`faction', lpattern(solid) lcolor(forest_green))
(rcap hi_`faction' lo_`faction' year_sec, lcolor(gs6))
	   ,
title(`faction')
xtitle("")
xscale(range(1967 2014) noextend)
xlabel(1967(5)2014,angle(45)) 
ylab(, nogrid)
graphregion(color(white))
legend(   off )
;
#delimit cr
graph save promotion_`faction', replace
}

// Princelings
local factionlist  Princelings 
foreach faction of local factionlist {
quietly: sum _b_`faction'
local ymin 0
local ymax .1

#delimit ;
twoway 
(scatteri `ymin' `=2012' `ymin'  `=2014', bcolor(gs15) recast(area) )
(scatteri `ymax' `=2012' `ymax'  `=2014', bcolor(gs15) recast(area) )
(tsline  _b_`faction', lpattern(solid) lcolor(dkorange))
(rcap hi_`faction' lo_`faction' year_sec, lcolor(gs6))
 ,
title(`faction')
xtitle("")
xscale(range(1967 2014) noextend)
xlabel(1967(5)2014,angle(45)) 
ylab(, nogrid)
graphregion(color(white))
legend(   off )
;
#delimit cr
graph save promotion_`faction', replace
}

graph combine  promotion_CYLC.gph promotion_Shanghai.gph promotion_Military.gph promotion_Princelings.gph, graphregion(color(white))
graph export  "D:\Dropbox\Chinese Politics\paper\table_figure\promotion.pdf", replace 


*************************
* regression
*************************


// promotion and retirement faction interact with general secretary and PBSC
set more off
local factionlist  CYLC Shanghai  Military Princelings 
foreach faction of local factionlist  {
use CC_8_18_nogap, clear
collapse (sum) Score CC AC PB PBSC, by(`faction' year)
keep if `faction'==1
save `faction', replace
}
clear

// total
use CC_8_18_nogap, clear
collapse (sum) Score CC AC PB PBSC, by( year)
local measurelist Score CC AC PB PBSC
foreach measure of local measurelist  {
rename `measure' sum_`measure'
}
save total, replace

foreach faction of local factionlist {
// CYLC faction share
use `faction', clear
merge 1:1 year using total
drop _m
local measurelist Score CC AC  PB  PBSC
foreach measure of local measurelist  {
replace `measure'=`measure'/sum_`measure'
rename `measure' `faction'_`measure'_share
}
keep year *_share
save `faction'_share, replace
}
clear

use CYLC_share, clear
local factionlist   Shanghai  Military Princelings 
foreach faction of local factionlist  {
merge 1:1 year using `faction'_share
drop _m
}
save faction_share, replace

use promotion_CC_above_same_below, clear
merge m:1 year using faction_share
drop if _m==2
drop _m

local factionlist  CYLC Shanghai  Military Princelings 
foreach faction of local factionlist  {
g `faction'_`faction'_PB = `faction'*`faction'_PB_share
g `faction'_`faction'_SC = `faction'*`faction'_PBSC_share
}

lab var CYLC_CYLC_sec "CYLC*CYLC leader"
lab var Shanghai_Shanghai_sec "Shanghai*Shanghai leader"
lab var Princelings_Princelings_sec "Princelings*Princelings leader"
lab var Military_Military_sec "Military*Military leader"
lab var CYLC_CYLC_SC "CYLC*CYLC SC share"
lab var Shanghai_Shanghai_SC "Shanghai*Shanghai SC share"
lab var Princelings_Princelings_SC "Princelings*Princelings SC share"
lab var Military_Military_SC "Military*Military SC share"
lab var CYLC "CYLC"
lab var Shanghai "Shanghai"
lab var Princelings "Princelings"
lab var Military "Military"
save  promotion_regression_sample, replace


 
*****************************************
* unconditional faction premia (used by CC/AC premia)
use promotion_regression_sample, clear
set more off

eststo clear
local y dum_promotion
local x CYLC  Shanghai  Princelings  Military 
local contr  gender college_dum gschool_dum mishu_dum ethnic_dum abroad_dum agez agez2 agez3    
local c ladder>1 & CC_no>=11
* All
quiet:eststo All1: reghdfe `y' `x'  `contr'  if `c', ab(ladder year)
estadd local indi "Yes"
estadd local FE1 "Yes"
estadd local FE "Yes"
estadd local sample "All"
predict y,xb 
sum y if CYLC==0&Shanghai==0&  Military==0& Princelings==0 
g p_All_Neutral = `r(mean)'
drop y
local factionlist  CYLC Shanghai  Military Princelings 
foreach faction of local factionlist  {
g p_All_`faction' = _b[`faction']
}
* AC
quiet:eststo All4: reghdfe `y' `x'  `contr'  if  `c' & ladder == 4, ab(ladder year) 
estadd local indi "Yes"
estadd local FE1 "Yes"
estadd local FE "Yes"
estadd local sample "AC"
predict y,xb 
sum y if CYLC==0&Shanghai==0&  Military==0& Princelings==0 
g p_AC_Neutral = `r(mean)'
drop y
local factionlist  CYLC Shanghai  Military Princelings 
foreach faction of local factionlist  {
g p_AC_`faction' = _b[`faction']
}
* CC
quiet:eststo All3: reghdfe `y' `x'  `contr'  if  `c' & ladder == 3, ab(ladder year) 
estadd local indi "Yes"
estadd local FE1 "Yes"
estadd local FE "Yes"
estadd local sample "CC"
predict y,xb 
sum y if CYLC==0&Shanghai==0&  Military==0& Princelings==0 
g p_CC_Neutral = `r(mean)'
drop y
local factionlist  CYLC Shanghai  Military Princelings 
foreach faction of local factionlist  {
g p_CC_`faction' = _b[`faction']
}
esttab
esttab using "./tmp/promotion_no_interaction.tex", ///
keep(CYLC  Shanghai  Princelings  Military ) ///
s(sample indi FE1 FE  N r2_a, label("Sample"  "Individual Attributes"  "Level F.E." "Year F.E." "Observations" "Adj. R-squared") ///
 fmt( 0 0 0 0  0 %9.2f ) )  mtitles("Promotion" "Promotion" "Promotion")  ///
width(\hsize) se star(* 0.1 ** 0.05 *** 0.01) ar2  br lab  replace nocon nonotes

keep p_*
keep if _n==1
xpose, clear  varname
sort _var
outsheet using  "D:\Dropbox\Chinese Politics\code\moment\promtion_prob_unconditional.csv", comma nolabel replace


*****************************************
* unconditional faction premia (used by CC/AC premia) - measure v2
use promotion_regression_sample, clear
set more off

eststo clear
local y dum_promotion
local x CYLC  Shanghai  Princelings  Military 
local contr  gender college_dum gschool_dum mishu_dum ethnic_dum abroad_dum agez agez2 agez3    
local c ladder>1 & CC_no>=11
* All
quiet:eststo All1: reghdfe `y' `x'  `contr'  if `c', ab(ladder year)
estadd local indi "Yes"
estadd local FE1 "Yes"
estadd local FE "Yes"
estadd local sample "All"
predict y,xb 
sum y if CYLC==0&Shanghai==0&  Military==0& Princelings==0 
g p_All_Neutral = `r(mean)'
local factionlist  CYLC Shanghai  Military Princelings 
foreach faction of local factionlist  {
sum y if `faction'==1
g p_All_`faction' = `r(mean)'
}
drop y
* AC
quiet:eststo All4: reghdfe `y' `x'  `contr'  if  `c' & ladder == 4, ab(ladder year) 
estadd local indi "Yes"
estadd local FE1 "Yes"
estadd local FE "Yes"
estadd local sample "AC"
predict y,xb 
sum y if CYLC==0&Shanghai==0&  Military==0& Princelings==0 
g p_AC_Neutral = `r(mean)'
local factionlist  CYLC Shanghai  Military Princelings 
foreach faction of local factionlist  {
sum y if `faction'==1
g p_AC_`faction' = `r(mean)'
}
drop y
* CC
quiet:eststo All3: reghdfe `y' `x'  `contr'  if  `c' & ladder == 3, ab(ladder year) 
estadd local indi "Yes"
estadd local FE1 "Yes"
estadd local FE "Yes"
estadd local sample "CC"
predict y,xb 
sum y if CYLC==0&Shanghai==0&  Military==0& Princelings==0 
g p_CC_Neutral = `r(mean)'
local factionlist  CYLC Shanghai  Military Princelings 
foreach faction of local factionlist  {
sum y if `faction'==1
g p_CC_`faction' = `r(mean)'
}
drop y

keep p_*
keep if _n==1
xpose, clear  varname
sort _var
outsheet using  "D:\Dropbox\Chinese Politics\code\moment/promtion_prob_unconditional_v2.csv", comma nolabel replace



*****************************************
* Conditional faction premian (Table 3)
use promotion_regression_sample, clear
keep if ladder>1 & CC_no>=11 & CC_no<=17
g cons=1

eststo clear
local y dum_promotion
local x CYLC CYLC_CYLC_sec CYLC_CYLC_SC Shanghai Shanghai_Shanghai_sec Shanghai_Shanghai_SC ///
Princelings Princelings_Princelings_SC Princelings_Princelings_sec ///
 Military Military_Military_SC Military_Military_sec
local contr  gender college_dum gschool_dum mishu_dum ethnic_dum abroad_dum agez agez2 agez3    
local c ladder>1 & CC_no>=11
* All
eststo All1: reghdfe `y' `x'  `contr'  if `c', ab(ladder year )
estadd local indi "Yes"
estadd local FE1 "Yes"
estadd local FE "Yes"
estadd local sample "All"
predict y,xb 
sum y if CYLC==0&Shanghai==0&  Military==0& Princelings==0 
g p_All_Neutral = `r(mean)'
drop y
local factionlist  CYLC Shanghai  Military Princelings 
foreach faction of local factionlist  {
g p_All_`faction' = _b[`faction']
g p_All_`faction'_SC = _b[`faction'_`faction'_SC]
g p_All_`faction'_sec = _b[`faction'_`faction'_sec]
}


* AC
eststo All2: reghdfe `y' `x'  `contr'  if  `c' , ab(ladder) 
estadd local indi "Yes"
estadd local FE1 "Yes"
estadd local FE "No"
estadd local sample "All"
predict y,xb 
sum y if CYLC==0&Shanghai==0&  Military==0& Princelings==0 
g p_AC_Neutral = `r(mean)'
drop y
local factionlist  CYLC Shanghai  Military Princelings 
foreach faction of local factionlist  {
g p_AC_`faction' = _b[`faction']
g p_AC_`faction'_SC = _b[`faction'_`faction'_SC]
g p_AC_`faction'_sec = _b[`faction'_`faction'_sec]
}
* CC
eststo All3: reghdfe `y' `x'  `contr'  if  `c' , ab(cons) 
estadd local indi "Yes"
estadd local FE1 "No"
estadd local FE "No"
estadd local sample "All"
predict y,xb 
sum y if CYLC==0&Shanghai==0&  Military==0& Princelings==0 
g p_CC_Neutral = `r(mean)'
drop y
local factionlist  CYLC Shanghai  Military Princelings 
foreach faction of local factionlist  {
g p_CC_`faction' = _b[`faction']
g p_CC_`faction'_SC = _b[`faction'_`faction'_SC]
g p_CC_`faction'_sec = _b[`faction'_`faction'_sec]
}
esttab All3 All2 All1
esttab All3 All2 All1 using "D:\Dropbox\Chinese Politics\paper\table_figure\promotion_interaction.tex", ///
keep( CYLC_CYLC_sec CYLC_CYLC_SC   Shanghai_Shanghai_sec Shanghai_Shanghai_SC  ///
  Princelings_Princelings_sec Princelings_Princelings_SC  ///
  Military_Military_sec Military_Military_SC ) ///
  order( CYLC_CYLC_sec CYLC_CYLC_SC   Shanghai_Shanghai_sec Shanghai_Shanghai_SC  ///
  Princelings_Princelings_sec Princelings_Princelings_SC  ///
  Military_Military_sec Military_Military_SC ) ///
s(sample indi FE1 FE  N r2_a, label("Sample"  "Individual Attributes"  "Level F.E." "Year F.E." "Observations" "Adj. R-squared") ///
 fmt( 0 0 0 0  0 %9.2f ) )  mtitles("Promotion" "Promotion" "Promotion")  ///
width(\hsize) se star(* 0.1 ** 0.05 *** 0.01) ar2  br lab  replace nocon nonotes


* moments on R2
/*
use promotion_regression_sample, clear
keep if ladder>1 & CC_no>=11 & CC_no<=17
g cons=1
*/

local x CYLC CYLC_CYLC_sec CYLC_CYLC_SC Shanghai Shanghai_Shanghai_sec Shanghai_Shanghai_SC ///
Princelings Princelings_Princelings_SC Princelings_Princelings_sec ///
 Military Military_Military_SC Military_Military_sec
local contr  gender college_dum gschool_dum mishu_dum ethnic_dum abroad_dum 
local c ladder>1 & CC_no>=11  

reghdfe dum_promotion agez agez2 agez3 if `c', ab(FE_LADDER=i.ladder FE_YEAR=i.year)
predict residual_prob, res

reg residual_prob `x'   if `c'
g faction_r2=e(r2)
reg residual_prob `contr'   if `c'
g ability_r2=e(r2)


keep p_* faction_r2 ability_r2
keep if _n==1
xpose, clear  varname
sort _var
outsheet using  "D:\Dropbox\Chinese Politics\code\moment\promtion_prob_conditional.csv", comma nolabel replace



* average share (5 groups, 3levels)
use promotion_CC_above_same_below, clear
keep if CC_no>=11
capture: drop f
g f = "Neutral"
local factionlist  CYLC Shanghai  Military Princelings 
foreach faction of local factionlist  {
replace f="`faction'" if `faction'==1
}


g seats = 1

g ladder_group = "PB" if ladder<=2 // in PB
replace ladder_group ="CC" if ladder==3
replace ladder_group ="AC" if ladder==4

bysort CC_no ladder_group: egen total_seats_by_level = sum(seats)
collapse (sum) seats (mean) total_seats_by_level, by(CC_no ladder_group f)
egen id = group(CC_no ladder_group)
encode f, g(f_n)
tsset id f_n
tsfill, full
carryforward CC_no f ladder_group total_seats_by_level, replace
replace seats=0 if seats==.
g seat_share = seats/total_seats_by_level
collapse (mean) seat_share , by(f ladder_group)
outsheet using  "D:\Dropbox\Chinese Politics\code\moment\moment_seat_share_5groups3levels.csv", comma nolabel replace


* average share (5 groups, populaton)
use promotion_CC_above_same_below, clear
keep if CC_no>=11
capture: drop f
g f = "Neutral"
local factionlist  CYLC Shanghai  Military Princelings 
foreach faction of local factionlist  {
replace f="`faction'" if `faction'==1
}


g seats = 1

g ladder_group = "All" 

bysort CC_no ladder_group: egen total_seats_by_level = sum(seats)
collapse (sum) seats (mean) total_seats_by_level, by(CC_no ladder_group f)
egen id = group(CC_no ladder_group)
encode f, g(f_n)
tsset id f_n
tsfill, full
carryforward CC_no f ladder_group total_seats_by_level, replace
replace seats=0 if seats==.
g seat_share = seats/total_seats_by_level
collapse (mean) seat_share , by(f ladder_group)
outsheet using  "D:\Dropbox\Chinese Politics\code\moment\moment_seat_share_5groups_population.csv", comma nolabel replace



// Summary states of promotion and term limit
use promotion_CC, clear
keep if  CC_no>=11
tabout promotion if promotion>0 using "D:\Dropbox\Chinese Politics\paper\table_figure\promotion_frequency.tex", ///
 percent replace style(tex) cells(freq col cum) clab(No. Col_% Cum_%)


g prom_retire = "No change" if dum_promotion==0
replace prom_retire="Promotion" if dum_promotion==1
replace prom_retire="Demotion" if promotion<0
replace prom_retire="Retirement" if retire==1

keep if  CC_no>=11
tabout prom_retire  using "D:\Dropbox\Chinese Politics\paper\table_figure\prom_retire_frequency.tex", ///
 percent replace style(tex) cells(freq col cum) clab(No. Col_% Cum_%) sort


bysort id ladder: g term = _N if CC_no>=11

keep name id ladder term
duplicates drop
replace term = 3 if term >3 & term!=.
tab term
tabout term using "D:\Dropbox\Chinese Politics\paper\table_figure\term_frequency.tex", ///
 percent replace style(tex) cells(freq col cum) clab(No. Col_% Cum_%)


 
 
******************************
* Robustness: unconditional faction premia (used by CC/AC premia) post 1992
*****************************************
use promotion_regression_sample, clear
set more off

eststo clear
local y dum_promotion
local x CYLC  Shanghai  Princelings  Military 
local contr  gender college_dum gschool_dum mishu_dum ethnic_dum abroad_dum agez agez2 agez3    
local c ladder>1 & CC_no>=14
* All
quiet:eststo All1: reghdfe `y' `x'  `contr'  if `c', ab(ladder year)
estadd local indi "Yes"
estadd local FE1 "Yes"
estadd local FE "Yes"
estadd local sample "All"
predict y,xb 
sum y if CYLC==0&Shanghai==0&  Military==0& Princelings==0 
g p_All_Neutral = `r(mean)'
drop y
local factionlist  CYLC Shanghai  Military Princelings 
foreach faction of local factionlist  {
g p_All_`faction' = _b[`faction']
}
* AC
quiet:eststo All4: reghdfe `y' `x'  `contr'  if  `c' & ladder == 4, ab(ladder year) 
estadd local indi "Yes"
estadd local FE1 "Yes"
estadd local FE "Yes"
estadd local sample "AC"
predict y,xb 
sum y if CYLC==0&Shanghai==0&  Military==0& Princelings==0 
g p_AC_Neutral = `r(mean)'
drop y
local factionlist  CYLC Shanghai  Military Princelings 
foreach faction of local factionlist  {
g p_AC_`faction' = _b[`faction']
}
* CC
quiet:eststo All3: reghdfe `y' `x'  `contr'  if  `c' & ladder == 3, ab(ladder year) 
estadd local indi "Yes"
estadd local FE1 "Yes"
estadd local FE "Yes"
estadd local sample "CC"
predict y,xb 
sum y if CYLC==0&Shanghai==0&  Military==0& Princelings==0 
g p_CC_Neutral = `r(mean)'
drop y
local factionlist  CYLC Shanghai  Military Princelings 
foreach faction of local factionlist  {
g p_CC_`faction' = _b[`faction']
}
esttab
esttab using "D:\Dropbox\Chinese Politics\paper\table_figure\promotion_no_interaction_14_17.tex", ///
keep(CYLC  Shanghai  Princelings  Military ) ///
s(sample indi FE1 FE  N r2_a, label("Sample"  "Individual Attributes"  "Level F.E." "Year F.E." "Observations" "Adj. R-squared") ///
 fmt( 0 0 0 0  0 %9.2f ) )  mtitles("Promotion" "Promotion" "Promotion")  ///
width(\hsize) se star(* 0.1 ** 0.05 *** 0.01) ar2  br lab  replace nocon nonotes


*******************************
* Robustness: Conditional faction premian (Table 3), post 1992 (CC_no 14)
use promotion_regression_sample, clear
keep if ladder>1 & CC_no>=14 & CC_no<=17
g cons=1

eststo clear
local y dum_promotion
local x CYLC CYLC_CYLC_sec CYLC_CYLC_SC Shanghai Shanghai_Shanghai_sec Shanghai_Shanghai_SC ///
Princelings Princelings_Princelings_SC Princelings_Princelings_sec ///
 Military Military_Military_SC Military_Military_sec
local contr  gender college_dum gschool_dum mishu_dum ethnic_dum abroad_dum agez agez2 agez3    
local c ladder>1 & CC_no>=11
* All
eststo All1: reghdfe `y' `x'  `contr'  if `c', ab(ladder year )
estadd local indi "Yes"
estadd local FE1 "Yes"
estadd local FE "Yes"
estadd local sample "All"
predict y,xb 
sum y if CYLC==0&Shanghai==0&  Military==0& Princelings==0 
g p_All_Neutral = `r(mean)'
drop y
local factionlist  CYLC Shanghai  Military Princelings 
foreach faction of local factionlist  {
g p_All_`faction' = _b[`faction']
g p_All_`faction'_SC = _b[`faction'_`faction'_SC]
g p_All_`faction'_sec = _b[`faction'_`faction'_sec]
}


* AC
eststo All2: reghdfe `y' `x'  `contr'  if  `c' , ab(ladder) 
estadd local indi "Yes"
estadd local FE1 "Yes"
estadd local FE "No"
estadd local sample "All"
predict y,xb 
sum y if CYLC==0&Shanghai==0&  Military==0& Princelings==0 
g p_AC_Neutral = `r(mean)'
drop y
local factionlist  CYLC Shanghai  Military Princelings 
foreach faction of local factionlist  {
g p_AC_`faction' = _b[`faction']
g p_AC_`faction'_SC = _b[`faction'_`faction'_SC]
g p_AC_`faction'_sec = _b[`faction'_`faction'_sec]
}
* CC
eststo All3: reghdfe `y' `x'  `contr'  if  `c' , ab(cons) 
estadd local indi "Yes"
estadd local FE1 "No"
estadd local FE "No"
estadd local sample "All"
predict y,xb 
sum y if CYLC==0&Shanghai==0&  Military==0& Princelings==0 
g p_CC_Neutral = `r(mean)'
drop y
local factionlist  CYLC Shanghai  Military Princelings 
foreach faction of local factionlist  {
g p_CC_`faction' = _b[`faction']
g p_CC_`faction'_SC = _b[`faction'_`faction'_SC]
g p_CC_`faction'_sec = _b[`faction'_`faction'_sec]
}
esttab All3 All2 All1
esttab All3 All2 All1 using "D:\Dropbox\Chinese Politics\paper\table_figure\promotion_interaction_14_17.tex", ///
keep( CYLC_CYLC_sec CYLC_CYLC_SC   Shanghai_Shanghai_sec Shanghai_Shanghai_SC  ///
  Princelings_Princelings_sec Princelings_Princelings_SC  ///
  Military_Military_sec Military_Military_SC ) ///
  order( CYLC_CYLC_sec CYLC_CYLC_SC   Shanghai_Shanghai_sec Shanghai_Shanghai_SC  ///
  Princelings_Princelings_sec Princelings_Princelings_SC  ///
  Military_Military_sec Military_Military_SC ) ///
s(sample indi FE1 FE  N r2_a, label("Sample"  "Individual Attributes"  "Level F.E." "Year F.E." "Observations" "Adj. R-squared") ///
 fmt( 0 0 0 0  0 %9.2f ) )  mtitles("Promotion" "Promotion" "Promotion")  ///
width(\hsize) se star(* 0.1 ** 0.05 *** 0.01) ar2  br lab  replace nocon nonotes
 
