# -*- coding: utf-8 -*-
"""
Created on Sun Jul 25 16:18:54 2021

@author: zhuoli
"""


import copy
import time
import pandas as pd
import random
import numpy as np
from collections import Counter
from matplotlib.backends.backend_pdf import PdfPages
import matplotlib.pyplot as plt
import matplotlib as mlp
from scipy.stats import mstats
mlp.rcParams['figure.dpi'] = 300
mlp.rcParams['figure.figsize'] = (8,4)
mlp.rcParams['axes.titlesize'] = 15
mlp.rcParams['axes.labelsize'] = 12


from .parameters import *


#####################
#   Read data
#####################
def readdata(file,having_CC=None):
    classiccol=["number","type", "leader","econ","misecon", 
                "CYLC","Shanghai","Princelings","Military","Neutral",
                "CYLC_PB","Shanghai_PB","Princelings_PB","Military_PB","Neutral_PB"]
    data = pd.DataFrame(columns=classiccol)
    with open(file) as fp:
        line = fp.readline()
        while line:
            if "top leader" in line:
                line=line.strip().split(":")
                nbr=int(line[0])
                types=line[1]
                leader=int(line[3])
                
                line = fp.readline()
                line=line.strip().split(":")
                types=line[1]
                tmpdis =line[3].strip().strip("[]").split()
                distribution=[int(i) for i in tmpdis]
                
                line = fp.readline()
                line=line.strip().split(":")
                types=line[1]
                tmpdis =line[3].strip().strip("[]").split()
                PBdistribution=[int(i) for i in tmpdis]
                
                line = fp.readline()
                line=line.strip().split(":")
                types=line[1]
                econvalue = float(line[3].strip())

                line = fp.readline()
                line=line.strip().split(":")
                types=line[1]
                misecon = float(line[3].strip())

                tmp_pd= pd.DataFrame([[nbr,types,leader,econvalue,misecon]+distribution+PBdistribution],columns=classiccol)
                data=pd.concat([data,tmp_pd],axis=0)
            line = fp.readline()
    
    
#     for each in ["CYLC","Shanghai","Princelings","Military","Neutral"]:
#         data[each+"_ACCC"]=data[each]-data[each+"_PB"]

    data['timeline']=data.groupby(['number','type']).cumcount()-100
    return data




def represent_ratio(data,fraction0):
    representativeness=1-np.sum((data[["CYLC","Shanghai","Princelings","Military"]]-fraction0[:-1]).abs(),axis=1)
    return representativeness

def represent_ratio_allfactions(data,fraction0):
    fraction0=np.array(fraction0)
    representativeness=1-np.sum((data[["CYLC","Shanghai","Princelings","Military","Neutral"]]-fraction0).abs(),axis=1)
    return representativeness

def represent_ratio_allfactions_PB(data,fraction0):
    fraction0=np.array(fraction0)
    representativeness=1-np.sum((data[["CYLC_PB","Shanghai_PB","Princelings_PB","Military_PB","Neutral_PB"]]-fraction0).abs(),axis=1)
    return representativeness

def absdiff(fraction_pd,lag):
    for each in ['CYLC','Shanghai','Princelings','Military','Neutral']:
        fraction_pd = fraction_pd.sort_values(by=['number','type','timeline'])
        fraction_pd.loc[:,each+'_diff'+str(lag)]=fraction_pd.groupby(['number','type'])[each].diff(lag).abs()
    for each in ["CYLC_PB","Shanghai_PB","Princelings_PB","Military_PB","Neutral_PB"]:
        fraction_pd = fraction_pd.sort_values(by=['number','type','timeline'])
        fraction_pd.loc[:,each+'_diff'+str(lag)]=fraction_pd.groupby(['number','type'])[each].diff(lag).abs()
    return fraction_pd



def total_pd(data):
    fraction0 = [0.045, 0.035, 0.042, 0.239, 0.639]
    
    data['total'] =      data[["CYLC", "Shanghai","Princelings", "Military", "Neutral"]].sum(axis=1)
    data['total_PB'] =   data[["CYLC_PB", "Shanghai_PB", "Princelings_PB", "Military_PB", "Neutral_PB"]].sum(axis=1)
    
    for faction in ["CYLC", "Shanghai", "Princelings", "Military", "Neutral"]:
        data[faction] = data[faction]/data['total']
    for faction in ["CYLC", "Shanghai", "Princelings", "Military", "Neutral"]:
        data[faction+"_PB"] = data[faction+"_PB"]/data['total_PB']
        
    data.loc[:, 'representativeness_noNeutral'] = represent_ratio(data, fraction0)
    data.loc[:, 'representativeness_allfactions'] = represent_ratio_allfactions(data, fraction0)
    data.loc[:, 'representativeness_allfactions_PB'] = represent_ratio_allfactions_PB(data, fraction0)
    data = absdiff(data, 1)
    data = absdiff(data, 2)

    data.loc[:, 'short-term instability_noNeutral'] =  ((data[[col for col in data.columns if '_diff1' in col]] \
                                                         [["CYLC_diff1", "Shanghai_diff1", "Princelings_diff1", "Military_diff1"]] \
                                                        )).sum(axis=1,skipna=False)
    
    data.loc[:, 'long-term instability_noNeutral'] =   ((data[[col for col in data.columns if '_diff2' in col]] \
                                                        [["CYLC_diff2", "Shanghai_diff2", "Princelings_diff2", "Military_diff2"]] \
                                                        )).sum(axis=1,skipna=False)

    data.loc[:, 'short-term instability_noNeutral_PB'] =  ((data[[col for col in data.columns if '_diff1' in col]] \
                                                         [["CYLC_PB_diff1", "Shanghai_PB_diff1", "Princelings_PB_diff1", \
                                                           "Military_PB_diff1"]])).sum(axis=1,skipna=False)
    
    data.loc[:, 'long-term instability_noNeutral_PB'] =   ((data[[col for col in data.columns if '_diff2' in col]] \
                                                        [["CYLC_PB_diff2", "Shanghai_PB_diff2", "Princelings_PB_diff2", \
                                                          "Military_PB_diff2"]])).sum(axis=1,skipna=False)
    
    data.loc[:, 'short-term instability_allfactions'] =  ((data[[col for col in data.columns if '_diff1' in col]] \
                                                         [["CYLC_diff1", "Shanghai_diff1", "Princelings_diff1", "Military_diff1", \
                                                           "Neutral_diff1"]])).sum(axis=1,skipna=False)
    
    data.loc[:, 'long-term instability_allfactions'] =  ((data[[col for col in data.columns if '_diff2' in col]] \
                                                        [["CYLC_diff2", "Shanghai_diff2", "Princelings_diff2", "Military_diff2", \
                                                          "Neutral_diff2"]])).sum(axis=1,skipna=False)
    
    data.loc[:, 'short-term instability_allfactions_PB'] = ((data[[col for col in data.columns if '_diff1' in col]] \
                                                         [["CYLC_PB_diff1", "Shanghai_PB_diff1", "Princelings_PB_diff1", \
                                                           "Military_PB_diff1","Neutral_PB_diff1"]])).sum(axis=1,skipna=False)
    
    data.loc[:, 'long-term instability_allfactions_PB'] = ((data[[col for col in data.columns if '_diff2' in col]] \
                                                        [["CYLC_PB_diff2", "Shanghai_PB_diff2", "Princelings_PB_diff2", \
                                                          "Military_PB_diff2","Neutral_PB_diff2"]])).sum(axis=1,skipna=False)
    
    
    rolling_lists = ['econ','CYLC', 'Shanghai', 'Princelings', 'Military','Neutral',
                     "CYLC_PB", "Shanghai_PB", "Princelings_PB", "Military_PB", "Neutral_PB",
                     'representativeness_noNeutral','representativeness_allfactions','representativeness_allfactions_PB',
                     'short-term instability_noNeutral', 'long-term instability_noNeutral',
                     'short-term instability_allfactions', 'long-term instability_allfactions',
                     'short-term instability_noNeutral_PB', 'long-term instability_noNeutral_PB',
                    'short-term instability_allfactions_PB','long-term instability_allfactions_PB']
    for eachvar in rolling_lists:
        data.loc[:, 'rolling_'+eachvar] = data.groupby(
            ['number', 'type'])[eachvar].transform(lambda x: x.rolling(10, 10).mean())

    return data



def plot_rollings(data0, i, version):

    #########################################
    # fraction_raw_
    #########################################
    # use Red for CYCL line, Blue for Shanghai, Yellow for Princelings, and Green for Military.
    figure, axes = plt.subplots(nrows=2, ncols=2)

    axes[0, 0].plot(data0[(data0['type'] == 'Baseline') & (data0.index >= 0)]['CYLC'], color='r', linewidth=0.5)
    axes[0, 0].set_title('CYLC', size='medium')

    axes[0, 1].plot(data0[(data0['type'] == 'Baseline') & (data0.index >= 0)]['Shanghai'], color='b', linewidth=0.5)
    axes[0, 1].set_title('Shanghai', size='medium')

    axes[1, 0].plot(data0[(data0['type'] == 'Baseline') & (data0.index >= 0)]['Princelings'], color='gold', linewidth=0.5)
    axes[1, 0].set_title('Princelings', size='medium')

    axes[1, 1].plot(data0[(data0['type'] == 'Baseline') & (data0.index >= 0)]['Military'], color='limegreen', linewidth=0.5)
    axes[1, 1].set_title('Military', size='medium')

    figure.tight_layout(pad=1.0)

    pp = PdfPages(folder+"\\Results\\Plots\\"+version +
                  "\\fraction_raw_" + str(i)+".pdf")
    pp.savefig()
    pp.close()
    plt.close()

    #########################################
    # fraction_rolling_PB_raw
    #########################################    
    # use Red for CYCL line, Blue for Shanghai, Yellow for Princelings, and Green for Military.
    figure, axes = plt.subplots(nrows=2, ncols=2)

    axes[0, 0].plot(data0[(data0['type'] == 'Baseline') & (data0.index >= 0)]['CYLC_PB'], color='r', linewidth=0.5)
    axes[0, 0].set_title('CYLC', size='medium')

    axes[0, 1].plot(data0[(data0['type'] == 'Baseline') & (data0.index >= 0)]['Shanghai_PB'], color='b', linewidth=0.5)
    axes[0, 1].set_title('Shanghai', size='medium')

    axes[1, 0].plot(data0[(data0['type'] == 'Baseline') & (data0.index >= 0)]['Princelings_PB'], color='gold', linewidth=0.5)
    axes[1, 0].set_title('Princelings', size='medium')

    axes[1, 1].plot(data0[(data0['type'] == 'Baseline') & (data0.index >= 0)]['Military_PB'], color='limegreen', linewidth=0.5)
    axes[1, 1].set_title('Military', size='medium')

    figure.tight_layout(pad=1.0)

    pp = PdfPages(folder+"\\Results\\Plots\\" +
                  version+"\\fraction_PB_raw_" + str(i)+".pdf")
    pp.savefig()
    pp.close()
    plt.close()
    return












































