# -*- coding: utf-8 -*-
"""
Created on Sun Jul 25 11:16:33 2021

@author: zhuoli
"""

import copy
import time
import pandas as pd
import random
import numpy as np
from collections import Counter
from scipy.stats import mstats

from .parameters import *

#########################
#   Constant
#########################
order = ['Faction premia: leader, CYLC',
         'Faction premia: leader, Shanghai',
         'Faction premia: leader, Military',
         'Faction premia: leader, Princelings',
         'Faction premia: SC share, CYLC',
         'Faction premia: SC share, Shanghai',
         'Faction premia: SC share, Princelings',
         'Faction premia: SC share, Military',
         'Faction premia: CC/AC ratio, CYLC',
         'Faction premia: CC/AC ratio, Shanghai',
         'Same-faction discount: national, CYLC',
         'Same-faction discount: national, Shanghai',
         'Same-faction discount: provincial, CYLC',
         'Same-faction discount: provincial, Shanghai',
         'Ability R-squared, total',
         'Promotion probability, CYLC',
         'Promotion probability, Shanghai',
         'Promotion probability, Princelings',
         'Promotion probability, Military',
         'Promotion probability, Neutral']


#########################
#   Weighting Matrix
#########################
def getdata(file):

    with open(file) as f:
        line=f.readlines()

    res=[]
    count=0
    diff=[]
    for row in line:
        row=row[2:].strip()
        if 'CYLC' in row:
            tmp=row.split("CYLC")
            contents=tmp[0].strip()
            data=[float(i) for i in tmp[1].split()]
            res.append([contents,"CYLC"]+data)
        if 'Shanghai' in row:
            tmp=row.split("Shanghai")
            contents=tmp[0].strip()
            data=[float(i) for i in tmp[1].split()]
            res.append([contents,"Shanghai"]+data)
        if 'Princelings' in row:
            tmp=row.split("Princelings")
            contents=tmp[0].strip()
            data=[float(i) for i in tmp[1].split()]
            res.append([contents,"Princelings"]+data)
        if 'Military' in row:
            tmp=row.split("Military")
            contents=tmp[0].strip()
            data=[float(i) for i in tmp[1].split()]
            res.append([contents,"Military"]+data)
        if 'Neutral' in row:
            tmp=row.split("Neutral")
            contents=tmp[0].strip()
            data=[float(i) for i in tmp[1].split()]
            res.append([contents,"Neutral"]+data)
        if 'total' in row:
            tmp=row.split("total")
            contents=tmp[0].strip()
            data=[float(i) for i in tmp[1].split()]
            res.append([contents,"total"]+data)

    res=pd.DataFrame(res).rename(columns={0:'contents',1:'faction',2:'true',3:'estimate'})
    
    return res


def using_mstats(s):
    return mstats.winsorize(s, limits=[0.05, 0.05])

def simulated_moments(file,version,num):
    res=[]
    file=Grid_data+"\\"+version+"\\moments\\"+file+"."
    true=getdata(file+str(1))[['contents','faction','true']]
    for i in range(1,num+1):
        tmp=getdata(file+str(i))
        tmp['nbr']=i
        res.append(tmp[['contents','faction','estimate','nbr']].set_index(['contents','faction']))
    data=pd.concat(res,axis=0).reset_index()
    data['estimate']=data.groupby(['contents','faction']).transform(lambda x: using_mstats(x))
    
    return data




def simulated_moments(file,version,num,subfolder="moments"):
    res=[]
    tmpfile=file
    tmpfile=Grid_data+"\\"+version+"\\"+subfolder+"\\"+tmpfile+"."
    true=getdata(tmpfile+str(1))[['contents','faction','true']]
    for i in range(1,num+1):
        tmp=getdata(tmpfile+str(i))
        res.append(tmp[['contents','faction','estimate']].set_index(['contents','faction']))


    data=pd.concat(res,axis=0).reset_index()
    data['estimate']=data.groupby(['contents','faction']).transform(lambda x: using_mstats(x))
    data=data.groupby(['contents','faction']).agg({'estimate':[np.mean,np.std]}).reset_index()
    data.columns=['contents','faction','estimated mean','estimated std']
    data=true.merge(data,on=['contents','faction'])
    data['diff']=(data['estimated mean']-data['true'])/data['estimated std']

    data['tmp']=', '
    data['Moments']=data['contents']+data['tmp']+data['faction']

    renamecol={'true':'Actual','estimated mean':'Simulated','estimated std':'S.E.'}
    outputpath=folder+"\\Results\\Tables\\"+version+"\\moments.tex"


    return data[['Moments','true','estimated mean','estimated std']].set_index('Moments'). \
        loc[order].rename(columns=renamecol).reset_index()





def changed_diags(df):

    t=[]
    df=[el.strip().strip("\\") for el in df]

    df[0]='0  '+df[0]

    for i in range(1,len(df)):
        tmpi=df[i]
        tmpi=tmpi.strip()
        tmpdf=tmpi.replace(' ','_')
        tmpres=tmpdf.replace('__',' ').split()
        tmp=tmpres[0]
        tmp+=' '
        tmp+=tmpres[1]
        for j in range(2,len(tmpres)):
            tmp+=' '
            tmp+=tmpres[j].replace('_',' ')  
        df[i]=tmp

    dfpd=[el.split()[1:] for el in df]
    df=pd.DataFrame(dfpd)

    df.columns=df.iloc[0]
    df=df.iloc[1:]
    df.loc[:,'contents']=df.contents.apply(lambda x:x.strip("_").replace("_"," ").strip())
    df['moments']=df.apply(lambda x: x['contents']+', '+x['faction'],axis=1)


    ps=['p'+str(i) for i in [0,1,2,3,5,6,7,8,9,10,11]]
    df=df[['moments']+ps]

    for k in ps:
        df[k]=df[k].astype('float')



    p=df.set_index('moments').loc[order][ps].values
    for i in range(p.shape[0]):
        p[i,:]=p[i,:]/weighting.loc[i]
    # c=np.dot(p.T,invcovmatrix)
    # c=np.dot(c,p)
    c=np.dot(p.T,p)
    # calculate inverse matrix
    d=np.linalg.inv(c)
    # take out the diagnoal terms and take square root
    diags=np.diag(d)**0.5

    return diags






def findline(line,strings):
    for i in range(len(line)):
        if strings in line[i]:
            return i
        

def read_hypotestdata(line,dfs,version,Notes=None,method=None):
    
    x0start=findline(line,'x0')+1
    x0end=findline(line,'diag')
    x0=[float(line[i].strip()) for i in range(x0start,x0end)]
    
    diagstart=findline(line,'diag')+1
    diagend=findline(line,'tstats')
    if method:
        diags=changed_diags(dfs)
    else:
        diags=[float(line[i].strip()) for i in range(diagstart,diagend)]
        
    tstart=findline(line,'tstats')+1
    tend=len(line)
    tstats=[float(line[i].strip()) for i in range(tstart,tend)]
    
    # take care of w_sum
    tree_num_child = np.array([1,9,3,3])    
    level_number=len(tree_num_child)+1
    num_bottom_level_node=np.prod(tree_num_child)
    nodes_number_by_level=np.cumprod(np.append(np.array([1]),tree_num_child[::-1],axis=0))
    politician_number_by_level=nodes_number_by_level*2

    w0,w1,w2=x0[4:7]
    w_by_level=np.array([w0,w0,w0,w0+w1,w0+w1])
    w_sum=np.sum(politician_number_by_level[::-1]*w_by_level)+w2
    diag1,diag2=diags[5:7]
    w0=w0/w_sum
    w1=w1/w_sum
    w2=w2/w_sum+w1
    diag1=diag1/w_sum
    diag2=(diag1**2+diag2**2)**0.5/w_sum
    x0[4]=w0
    x0[5]=w1
    x0[6]=w2
    diags[4]=diag1
    diags[5]=diag2


    # take care of alpha
    alpha=x0[7]
    for i in range(4):
        x0[i]=x0[i]/alpha
        diags[i]=diags[i]/alpha
    x0[7]=1
    diags[6]=diags[6]/alpha
    
    
    x0=x0[:4]+x0[5:]
    x0=np.array(x0).reshape(-1,1)
    diags=np.array(diags).reshape(-1,1)
    tstats=np.array(tstats).reshape(-1,1)
    
    res=np.append(x0,diags,axis=1)
    res=np.append(res,tstats,axis=1)
    res=pd.DataFrame(res).rename(columns={0:'Value',1:'S.E.',2:'t-stats'})

    Symbol=['$v_0$','$v_1$','$\delta_0$','$\delta_1$','$\omega_1$','$\omega_2$','$\alpha$'
            ,'$\theta_1$','$\theta_2$','$\theta_3$','$\theta_4$']
    Parameters=['Private benefits, intercept','Private benefits, slope','Utility gain, intercept','Utility gain, slope',
                'Support, SC members','Support, top leader','Alpha','Faction cohesion, CYLC','Faction cohesion, Shanghai',
               'Faction cohesion, Princelings','Faction cohesion, Military']
#     Symbol=['$v_0$','$v_1$','$\delta_0$','$\delta_1$','$\omega_1$','$\omega_2$','$\alpha$'
#             ,'$\theta_1$','$\theta_2$','$\theta_3$','$\theta_4$','$a_1$','$a_2$','$a_3$','$a_4$']
#     Parameters=['Private benefits, intercept','Private benefits, slope','Utility gain, intercept','Utility gain, slope',
#                 'Support, SC members','Support, top leader','Alpha','Faction cohesion, CYLC','Faction cohesion, Shanghai',
#                'Faction cohesion, Princelings','Faction cohesion, Military','Ability, CYLC','Ability, Shanghai',
#                 'Ability, Princelings','Ability, Military']
    res['t-stats']=res['Value']/res['S.E.']
    res=res.round(3)
    res['Symbol']=Symbol
    res['Parameters']=Parameters


#     order = ['Faction cohesion, CYLC', 'Faction cohesion, Shanghai','Faction cohesion, Princelings', 'Faction cohesion, Military',
#              'Ability, CYLC','Ability, Shanghai','Ability, Princelings','Ability, Military',
#             'Support, SC members', 'Support, top leader','Utility gain, intercept', 'Utility gain, slope',
#             'Private benefits, intercept', 'Private benefits, slope']

    order = ['Faction cohesion, CYLC', 'Faction cohesion, Shanghai','Faction cohesion, Princelings', 'Faction cohesion, Military',
            'Support, SC members', 'Support, top leader','Utility gain, intercept', 'Utility gain, slope',
            'Private benefits, intercept', 'Private benefits, slope']

    if Notes:
        outputpath2=folder+"\\Results\\Tables\\"+version+"\\parameters_"+Notes+".tex"
    else:
        outputpath=folder+"\\Results\\Tables\\"+version+"\\parameters.tex"
    

    res=res.set_index('Parameters').loc[order].reset_index()[['Symbol','Parameters','Value','S.E.']]
#     print(res.to_string())
    if Notes:
        res.to_latex(encoding='utf-8',index=False,escape=False,buf=outputpath2) 
    else:
        res.to_latex(encoding='utf-8',index=False,escape=False,buf=outputpath) 
        
    return res



def generate_res(file1,version,method=None):
    with open(Grid_data+"\\"+version+"\\"+file1) as f:
        line=f.readlines()   

    t=[]
    for i in range(len(line)):
        if 'p0' in line[i]: t.append(i)
        if 'diags' in line[i]:
            t.append(i)
        if 'tstats' in line[i]:
            t.append(i)
        if 'partial' in line[i]:
            t.append(i)
            break

    divide=[]
    for el in range(len(line)):
        if 'partial' in line[el]:
            tmp=line[el+1]  
            nbr=int(tmp.split()[0])
            divide.append(nbr)

    each=[]
    tmp=[]
    dfs=[]
    gapdiags=t[2]-t[1]
    gapt=t[3]-t[2]
    for el in range(len(line)):
        if '****x0****' in line[el]:
            tmp+=line[el:el+t[0]]
            dfs.append(line[el+t[0]:el+t[3]])
        if 'diags' in line[el]:
            tmp+=line[el:el+gapdiags]
        if 'tstats' in line[el]:
            tmp+=line[el:el+gapt]
            each.append(tmp)
            tmp=[]

            
    for eachdiv in range(len(divide)):
        if divide[eachdiv]==1000:
            Note='divided'+str(divide[eachdiv])
            read_hypotestdata(each[eachdiv],dfs[eachdiv],version,Notes=Note,method=method)










