# -*- coding: utf-8 -*-


import copy
import time
import pandas as pd
import random
import numpy as np
from collections import Counter
from scipy.stats import mstats


from .parameters import *
from .Graphs import *


##########################
#   Read data
##########################

def read_data(filepath):
    classiccol=["number","type", "leader","econ","misecon", 
                "CYLC","Shanghai","Princelings","Military","Neutral",
                "CYLC_PB","Shanghai_PB","Princelings_PB","Military_PB","Neutral_PB"]
    floatccol=["econ","misecon"]
    intccol=["number", "leader","CYLC","Shanghai","Princelings","Military","Neutral",
                "CYLC_PB","Shanghai_PB","Princelings_PB","Military_PB","Neutral_PB"]
    with open(filepath) as fp:
        lines = fp.readlines()        
    
    tmp=pd.DataFrame(lines)
    tmp[0]=tmp[0].apply(lambda x: x.strip("\n"))
    tmp['raw']=tmp[0].apply(lambda x: x.split(":"))
    
    tmp['relative_idx']=tmp.index
    tmp['nbr']=tmp['raw'].apply(lambda x: x[0])
    tmp['types']=tmp['raw'].apply(lambda x: x[1])
    tmp['content']=tmp['raw'].apply(lambda x: x[2])
    tmp['data']=tmp['raw'].apply(lambda x: x[3])
    
    tmp=tmp.sort_values(by=['types','relative_idx'])
    
    tmp['groupid']=np.mod(tmp.index,1000)
    tmp['group']=tmp.index//1000
    tmp['pivotid']=tmp.index//5
    tmp['pivotid2']=tmp.apply(lambda x: str(x['pivotid'])+","+x['nbr']+","+x['types'],axis=1)
    
    small=tmp[['nbr','types','content','data','pivotid','pivotid2']]
    small=small.pivot(index='pivotid2',columns='content',values='data').reset_index()
    small['pivotid2']=small.pivotid2.apply(lambda x:x.split(","))
    
    small['pivotid']=small['pivotid2'].apply(lambda x:x[0]).astype(int)
    small['nbr']=small['pivotid2'].apply(lambda x:x[1]).astype(int)
    small['types']=small['pivotid2'].apply(lambda x:x[2])

    small=small.sort_values(by=['nbr','pivotid'])[['nbr','pivotid','types','econ','top leader',
                                             'misecon','PB faction distribution','faction distribution']]

    small['faction distribution']=small['faction distribution'].apply(lambda x: x.strip().strip('[]').split())
    small['PB faction distribution']=small['PB faction distribution'].apply(lambda x: x.strip().strip('[]').split())


    small['CYLC']=small['faction distribution'].apply(lambda x: int(x[0]))
    small['Shanghai']=small['faction distribution'].apply(lambda x: int(x[1]))
    small['Princelings']=small['faction distribution'].apply(lambda x: int(x[2]))
    small['Military']=small['faction distribution'].apply(lambda x: int(x[3]))
    small['Neutral']=small['faction distribution'].apply(lambda x: int(x[4]))

    small['CYLC_PB']=small['PB faction distribution'].apply(lambda x: int(x[0]))
    small['Shanghai_PB']=small['PB faction distribution'].apply(lambda x: int(x[1]))
    small['Princelings_PB']=small['PB faction distribution'].apply(lambda x: int(x[2]))
    small['Military_PB']=small['PB faction distribution'].apply(lambda x: int(x[3]))
    small['Neutral_PB']=small['PB faction distribution'].apply(lambda x: int(x[4]))

    small.rename(columns={'nbr':'number','types':'type','top leader':'leader'},inplace=True)
    small=small[classiccol]
    
    for i in floatccol: small[i]=small[i].astype(float)
    for i in intccol: small[i]=small[i].astype(int)   
        
    small['timeline']=small.groupby(['number','type']).cumcount()-100
    
    return small





##########################
#   Ratios computation
##########################

def represent_ratio(data,fraction0):
    representativeness=1-np.sum((data[["CYLC","Shanghai","Princelings","Military"]]-fraction0[:-1]).abs(),axis=1)
    return representativeness

def represent_ratio_PB(data,fraction0):
    fraction0=np.array(fraction0)
    representativeness=1-np.sum((data[["CYLC_PB","Shanghai_PB","Princelings_PB","Military_PB"]]-fraction0[:-1]).abs(),axis=1)
    return representativeness

def represent_ratio_allfactions(data,fraction0):
    fraction0=np.array(fraction0)
    representativeness=1-np.sum((data[["CYLC","Shanghai","Princelings","Military","Neutral"]]-fraction0).abs(),axis=1)
    return representativeness

def represent_ratio_allfactions_PB(data,fraction0):
    fraction0=np.array(fraction0)
    representativeness=1-np.sum((data[["CYLC_PB","Shanghai_PB","Princelings_PB","Military_PB","Neutral_PB"]]-fraction0).abs(),axis=1)
    return representativeness

def absdiff(fraction_pd,lag):
    for each in ['CYLC','Shanghai','Princelings','Military','Neutral']:
        fraction_pd = fraction_pd.sort_values(by=['number','type','timeline'])
        fraction_pd.loc[:,each+'_diff'+str(lag)]=fraction_pd.groupby(['number','type'])[each].diff(lag).abs()
    for each in ["CYLC_PB","Shanghai_PB","Princelings_PB","Military_PB","Neutral_PB"]:
        fraction_pd = fraction_pd.sort_values(by=['number','type','timeline'])
        fraction_pd.loc[:,each+'_diff'+str(lag)]=fraction_pd.groupby(['number','type'])[each].diff(lag).abs()
    return fraction_pd



def total_pd_v2(data):
    fraction0 = [0.045, 0.035, 0.042, 0.239, 0.639]
    
    data['total'] =      data[["CYLC", "Shanghai","Princelings", "Military", "Neutral"]].sum(axis=1)
    data['total_PB'] =   data[["CYLC_PB", "Shanghai_PB", "Princelings_PB", "Military_PB", "Neutral_PB"]].sum(axis=1)
    
    for faction in ["CYLC", "Shanghai", "Princelings", "Military", "Neutral"]:
        data[faction] = data[faction]/data['total']
    for faction in ["CYLC", "Shanghai", "Princelings", "Military", "Neutral"]:
        data[faction+"_PB"] = data[faction+"_PB"]/data['total_PB']
        
    data.loc[:, 'representativeness_noNeutral'] = represent_ratio(data, fraction0)
    data.loc[:, 'representativeness_noNeutral_PB'] = represent_ratio_PB(data, fraction0)
    data.loc[:, 'representativeness_allfactions'] = represent_ratio_allfactions(data, fraction0)
    data.loc[:, 'representativeness_allfactions_PB'] = represent_ratio_allfactions_PB(data, fraction0)
    data = absdiff(data, 1)
    data = absdiff(data, 2)

    data.loc[:, 'short-term instability_noNeutral'] =  ((data[[col for col in data.columns if '_diff1' in col]] \
                                                          [["CYLC_diff1", "Shanghai_diff1", "Princelings_diff1", "Military_diff1"]] \
                                                        )).sum(axis=1,skipna=False)
    
    data.loc[:, 'long-term instability_noNeutral'] =   ((data[[col for col in data.columns if '_diff2' in col]] \
                                                        [["CYLC_diff2", "Shanghai_diff2", "Princelings_diff2", "Military_diff2"]] \
                                                        )).sum(axis=1,skipna=False)

    data.loc[:, 'short-term instability_noNeutral_PB'] =  ((data[[col for col in data.columns if '_diff1' in col]] \
                                                          [["CYLC_PB_diff1", "Shanghai_PB_diff1", "Princelings_PB_diff1", \
                                                            "Military_PB_diff1"]])).sum(axis=1,skipna=False)
    
    data.loc[:, 'long-term instability_noNeutral_PB'] =   ((data[[col for col in data.columns if '_diff2' in col]] \
                                                        [["CYLC_PB_diff2", "Shanghai_PB_diff2", "Princelings_PB_diff2", \
                                                          "Military_PB_diff2"]])).sum(axis=1,skipna=False)
    
    data.loc[:, 'short-term instability_allfactions'] =  ((data[[col for col in data.columns if '_diff1' in col]] \
                                                          [["CYLC_diff1", "Shanghai_diff1", "Princelings_diff1", "Military_diff1", \
                                                            "Neutral_diff1"]])).sum(axis=1,skipna=False)
    
    data.loc[:, 'long-term instability_allfactions'] =  ((data[[col for col in data.columns if '_diff2' in col]] \
                                                        [["CYLC_diff2", "Shanghai_diff2", "Princelings_diff2", "Military_diff2", \
                                                          "Neutral_diff2"]])).sum(axis=1,skipna=False)
    
    data.loc[:, 'short-term instability_allfactions_PB'] = ((data[[col for col in data.columns if '_diff1' in col]] \
                                                          [["CYLC_PB_diff1", "Shanghai_PB_diff1", "Princelings_PB_diff1", \
                                                            "Military_PB_diff1","Neutral_PB_diff1"]])).sum(axis=1,skipna=False)
    
    data.loc[:, 'long-term instability_allfactions_PB'] = ((data[[col for col in data.columns if '_diff2' in col]] \
                                                        [["CYLC_PB_diff2", "Shanghai_PB_diff2", "Princelings_PB_diff2", \
                                                          "Military_PB_diff2","Neutral_PB_diff2"]])).sum(axis=1,skipna=False)
    
    
    rolling_lists = ['econ','CYLC', 'Shanghai', 'Princelings', 'Military','Neutral',
                      "CYLC_PB", "Shanghai_PB", "Princelings_PB", "Military_PB", "Neutral_PB",
                      'representativeness_noNeutral','representativeness_allfactions','representativeness_allfactions_PB',
                      'short-term instability_noNeutral', 'long-term instability_noNeutral',
                      'short-term instability_allfactions', 'long-term instability_allfactions',
                    'short-term instability_allfactions_PB','long-term instability_allfactions_PB']
    for eachvar in rolling_lists:
        data.loc[:, 'rolling_'+eachvar] = data.groupby(
            ['number', 'type'])[eachvar].transform(lambda x: x.rolling(10, 10).mean())

    return data








##########################
#   Ratios computation
##########################
order = ['Baseline', 'Double leader premia','Dominant leader premia','All cohesion=0', 'CYLC cohesion=1','Shanghai cohesion=1', 'All cohesion=1', \
          'Half private benefits']
order_boot=[]
for i in order:
    order_boot.append(i)
    order_boot.append(i+'_b')


# renamecol = {'type': 'Counterfactuals', 'econ': 'Efficiency', \
#              'representativeness': 'Equality', 'long-term stability': 'Stability'}
renamecol2 = {'type': 'Counterfactuals', 'econ': 'Efficiency', \
             'representativeness': 'Malapportionment', 'short-term stability': 'Short-term Instability', \
              'long-term stability': 'Long-term Instability'}
renamecol_v5 = {'type': 'Counterfactuals', 'econ': 'Efficiency', \
             'representativeness_PB': 'Malapportionment', 'short-term stability_PB': 'Short-term Instability', \
                'long-term stability_PB': 'Long-term Instability'}
renamecol_bootstrap = {'Counterfactuals_b':'', 'Efficiency_b':'', \
             'representativeness_PB': 'Malapportionment', 'short-term stability_PB': 'Short-term Instability', \
                'long-term stability_PB': 'Long-term Instability'}
renameindex={'All 0':'All cohesion=0','All 1':'All cohesion=1','CYLC 1 other baseline':'CYLC cohesion=1', \
            'Shanghai 1 other baseline':'Shanghai cohesion=1','Double w2':'Double leader premia','Half v':'Half private benefits', \
            'Only w2':'Dominant leader premia'}

def renamecolumns(df):
    cols=df.columns
    for i in cols:
        if 'type' in i:
            df.rename(columns={i:'Counterfactuals'},inplace=True)
        if 'econ' in i:
            df.rename(columns={i:'Efficiency'},inplace=True)
        if 'representativeness' in i:
            if 'PB' in i: df.rename(columns={i:'Malapportionment (PB)'},inplace=True)
            if 'PB' not in i: df.rename(columns={i:'Malapportionment (AC)'},inplace=True)
        if 'stability' in i and 'short' in i:
            if 'PB' in i: df.rename(columns={i:'Short-term Instability (PB)'},inplace=True)
            if 'PB' not in i: df.rename(columns={i:'Short-term Instability (AC)'},inplace=True)
        if 'stability' in i and 'long' in i:
            if 'PB' in i: df.rename(columns={i:'Long-term Instability (PB)'},inplace=True)
            if 'PB' not in i: df.rename(columns={i:'Long-term Instability (AC)'},inplace=True)
                
    return df


def get_tmpmeanstds(stats,typelist):
    tmpmean = stats[['type']+typelist].groupby('type').mean() 
    tmpstd = stats[['type']+typelist].groupby('type').std() 
    return [tmpmean,tmpstd]







##########################
#   Bootstrap
##########################
def func_version4(tmpmean):
    drop1minus_mean=copy.deepcopy(tmpmean)
    for i in drop1minus_mean.columns:
        if 'represent' in i: drop1minus_mean[i]=1-drop1minus_mean[i]

    tmpprint4=100*drop1minus_mean/drop1minus_mean.loc['Baseline']-100
    return tmpprint4.rename(index=renameindex)

def func_version5(tmpmean):
    drop1minus_mean=copy.deepcopy(tmpmean)
    for i in drop1minus_mean.columns:
        if 'represent' in i: drop1minus_mean[i]=1-drop1minus_mean[i]

    tmpprint5=100*drop1minus_mean/drop1minus_mean.loc['Baseline']-100
    return tmpprint5.rename(index=renameindex)

def func_bootstrapdata(data,whichfunction,typelist,simulate_rounds=100):
    bootstrap=[]
    np.random.seed(0)
    for tmp_index in range(500):
        bootstrap_tmp=[np.random.randint(0,simulate_rounds) for _ in range(100)]
        tmpdata=data.set_index('number').loc[bootstrap_tmp]
        tmpstats = tmpdata[tmpdata['timeline'] > 0]
        tmpmean,tmpstd=get_tmpmeanstds(tmpstats,typelist)
        
        if whichfunction==4:
            tmpbootstrap=func_version4(tmpmean)
        if whichfunction==5:
            tmpbootstrap=func_version5(tmpmean)
        tmpbootstrap=tmpbootstrap.stack()
        tmpbootstrap=pd.DataFrame(tmpbootstrap).reset_index().rename(columns={0:'value', 'level_1':'content'})
        bootstrap.append(tmpbootstrap)
    return pd.concat(bootstrap,axis=0).sort_values(by=['type','content'])

def strfy(data):
    df=copy.deepcopy(data)
    df=df.astype(str)
    for i in data.index.values:
        for j in data.columns.values:
            p5=data.loc[i][j]
            strp5=str(p5) if len(str(p5).split('.')[-1])==3 else str(p5)+'0'*(3-len(str(p5).split('.')[-1]))
            df.loc[i][j]=strp5
    return df


def merge_data_bootstrap(data,bootstrapdata):
    # range from bootstrap for each index-column content
    data_bootstrap=dict()
    for i in data.index.values:
        for j in data.columns.values:
            a=bootstrapdata[(bootstrapdata['type']==i)& \
                         (bootstrapdata['content']==j)].value
            p5 = np.percentile(a, 5).round(3)
            p95 = np.percentile(a, 95).round(3)
            strp5=str(p5) if len(str(p5).split('.')[-1])==3 else str(p5)+'0'*(3-len(str(p5).split('.')[-1]))
            strp95=str(p95) if len(str(p95).split('.')[-1])==3 else str(p95)+'0'*(3-len(str(p95).split('.')[-1]))
            data_bootstrap[(i,j)]="["+strp5+", "+strp95+"]"

    data_bootstrap=pd.Series(data_bootstrap).reset_index()
    data_bootstrap=data_bootstrap.pivot(index='level_0', columns='level_1')[0].reset_index().rename(columns={'level_0':'Counterfactuals'})
    data_bootstrap=pd.DataFrame(data_bootstrap.to_records()).set_index('Counterfactuals')
    for i in data_bootstrap.index:
        data_bootstrap.rename(index={i:i+'_b'},inplace=True)
    data_bootstrap_index=data_bootstrap.index.values
    
    if 'index' in data_bootstrap: del data_bootstrap['index']
    data_bootstrap=pd.concat([strfy(data),data_bootstrap],axis=0,sort=False).loc[order_boot]
    for i in data_bootstrap_index:
        data_bootstrap.rename(index={i:''},inplace=True)    
    return data_bootstrap






##########################
#   various tables
##########################

def produce_tables_inSimulated_moments( data , stats , version ):

    
    ##############
    # Short-term
    ##############
    alist=['econ','representativeness_allfactions','representativeness_allfactions_PB',
             'short-term instability_allfactions','short-term instability_allfactions_PB']
    tmpmean,tmpstd=get_tmpmeanstds(stats,alist)
    
    # Fifth version
    # Include Neutral faction
    #   for stability and representativeness, change the formula by dropping "1-"
    #   (mean_counterfactual-mean_baseline) / mean_baseline  
    
    
    tmpprint=func_version5(tmpmean).round(3)
    bootstraps=func_bootstrapdata(data,5,alist)
    
    res=merge_data_bootstrap(tmpprint,bootstraps)
    res=renamecolumns(res).round(3)
    res.index.name='Counterfactuals'
    
    outputpath = folder+"\\Results\\Tables\\"+version+"\\counterfactuals_v5_short.tex"
    res.round(3).reset_index().to_latex(index=False, buf=outputpath)



    # Short-term baseline bootstrap
    bootstrap=[]
    np.random.seed(0)
    for _ in range(500):
        bootstrap_tmp=[np.random.randint(0,100) for _ in range(100)]
        tmpdata=data.set_index('number').loc[bootstrap_tmp]
        tmpstats = tmpdata[tmpdata['timeline'] > 0]
        tmpmean,tmpstd=get_tmpmeanstds(tmpstats,alist)
        tmpmean['representativeness_allfactions']=1-tmpmean['representativeness_allfactions']
        tmpmean['representativeness_allfactions_PB']=1-tmpmean['representativeness_allfactions_PB']
        tmpmean=tmpmean/2
        tmpbootstrap=tmpmean.loc['Baseline']
        tmpbootstrap=pd.DataFrame(tmpbootstrap)
        bootstrap.append(tmpbootstrap)
    resmean=pd.concat(bootstrap,axis=1)
    resmean.loc['econ']/=(resmean.loc['econ'].mean()/14.14)
    
    data=tmpmean.loc[['Baseline']].T
    data.loc['econ']=14.14
    bootstrapdata=copy.deepcopy(resmean)
    
    data_bootstrap=dict()
    for i in data.index.values:
        a=bootstrapdata.loc[i].values
        p5 = np.percentile(a, 5).round(3)
        p95 = np.percentile(a, 95).round(3)
        strp5=str(p5) if len(str(p5).split('.')[-1])==3 else str(p5)+'0'*(3-len(str(p5).split('.')[-1]))
        strp95=str(p95) if len(str(p95).split('.')[-1])==3 else str(p95)+'0'*(3-len(str(p95).split('.')[-1]))
        data_bootstrap[(i,'Baseline')]="["+strp5+", "+strp95+"]"
        
    data_bootstrap=pd.Series(data_bootstrap).reset_index()
    data_bootstrap=data_bootstrap.pivot(index='level_0', columns='level_1')[0].reset_index().rename(columns={'level_0':'Counterfactuals'})
    data_bootstrap=pd.DataFrame(data_bootstrap.to_records()).set_index('Counterfactuals')
    
    for i in data_bootstrap.index:
        data_bootstrap.rename(index={i:i+'_b'},inplace=True)
    data_bootstrap_index=data_bootstrap.index.values
    
    if 'index' in data_bootstrap: del data_bootstrap['index']
    res=pd.DataFrame(np.append(data.round(3).T.values,data_bootstrap.T.values,axis=0),columns=data.index)
    
    outputpath = folder+"\\Results\\Tables\\"+version+"\\Baseline_statistics_and_bootstrapCI.tex"
    res.round(3).reset_index().to_latex(index=False, buf=outputpath)








