# -*- coding: utf-8 -*-



import os
paths= r"C:\Users\zhuoli\Desktop\China_Project\Code\Simulation_estimation"
os.chdir(paths)

folder = r"C:\Users\zhuoli\Desktop\China_Project"

Grid_data=folder+"\\Grid Output"




import pandas as pd
import random
import numpy as np
from collections import Counter
import copy

import pickle
from numba import jit

from matplotlib.backends.backend_pdf import PdfPages
import matplotlib.pyplot as plt
import matplotlib as mlp
from scipy.stats import mstats

mlp.rcParams['figure.dpi'] = 300
mlp.rcParams['figure.figsize'] = (8,6)
mlp.rcParams['axes.titlesize'] = 15
mlp.rcParams['axes.labelsize'] = 12

from get_moments import *
from table_moments import *
from optimizer import *
from parameters import *



####################################
# Initialize tree
####################################
def initialize_tree(level,nodes_num,factiondistri,mustretires,factionmustretire):
    num_bottom_level_node=nodes_num
    
    # 2 dataframe to initialize: a,b are counterparts in the same level-tree
    # eg. node 1 at level0_a and node 1 at level0_b are partners
    level0_a=pd.DataFrame(pd.Series(range(num_bottom_level_node)))
    level0_b=pd.DataFrame(pd.Series(range(num_bottom_level_node)))
    level0_a=level0_a.rename(columns={0:"Node"})
    level0_b=level0_b.rename(columns={0:"Node"})
    level0_a['Node_v2']=level0_a['Node']
    level0_b['Node_v2']=level0_b['Node']  
    
    # left or right
    level0_a['LR']=0
    level0_b['LR']=1
    # final node
    level0_a['LR_v2']=0
    level0_b['LR_v2']=1
    
    # level
    level0_a['Level']=level
    level0_b['Level']=level
    # level
    level0_a['Level_v2']=level
    level0_b['Level_v2']=level
    
    # ability
    level0_a['Ability']=level0_a.apply(lambda x: np.random.uniform(0,10), axis=1)
    level0_b['Ability']=level0_a.apply(lambda x: np.random.uniform(0,10), axis=1)
    # faction
    tmp=[]
    tmp2=[]
    for i in range(len(factiondistri)):
        if len(tmp)+factiondistri[i]<=num_bottom_level_node:
            tmp+=[i]*factiondistri[i]
        else:
            break
    if tmp[-1]>=3:
        tmp2=[num_faction-1]*num_bottom_level_node
    else:
        for i in range(tmp[-1]+1,num_faction-1):
            tmp2+=[i]*factiondistri[i]
    tmp+=[num_faction-1]*(num_bottom_level_node-len(tmp))
    tmp2+=[num_faction-1]*(num_bottom_level_node-len(tmp2))
    level0_a['Faction']=tmp
    level0_b['Faction']=tmp2

    # birth
    maxf=np.max(level0_a[level0_a['Faction']<4]['Faction'].values)
    if maxf==4: maxf-=1        
    tmpage1=[]
    tmpage2=[]
    for i in range(maxf+1):
        tmpadd=[-1]*factionmustretire[i]+[0]*(factiondistri[i]-factionmustretire[i])
        np.random.shuffle(tmpadd)
        tmpage1+=tmpadd
    # if all factions are in left, which means nodes greatly larger than faction members
    if maxf<3:        
        for i in range(maxf+1,num_faction-1):
            tmpadd=[-1]*factionmustretire[i]+[0]*(factiondistri[i]-factionmustretire[i])
            np.random.shuffle(tmpadd)
            tmpage2+=tmpadd
    fnbr=np.sum(factiondistri)
    neutralnbr=2*num_bottom_level_node-fnbr
    tmpadd=[-1]*factionmustretire[-1]+[0]*(neutralnbr-factionmustretire[-1])
    np.random.shuffle(tmpadd)
    
    tmp1gap=num_bottom_level_node-len(tmpage1)
    tmpage1+=tmpadd[:tmp1gap]
    tmpage2+=tmpadd[tmp1gap:]

    level0_a['Birth']=tmpage1
    level0_b['Birth']=tmpage2
    
    level0_a=level0_a[['Birth','Level','LR','Node','Level_v2','LR_v2','Node_v2','Faction','Ability']]
    level0_b=level0_b[['Birth','Level','LR','Node','Level_v2','LR_v2','Node_v2','Faction','Ability']]
    
    return np.array(level0_a),np.array(level0_b)



def init_database(given_distribution,mustretire,factionmustretire,topleader):    
    tmpdata= initialize_tree(0, num_bottom_level_node,given_distribution[0],mustretire[0],factionmustretire[0])
    level0_a=tmpdata[0]
    level0_b=tmpdata[1]
    # all trees save in a list
    data=np.append(level0_a,level0_b,axis=0)
    
    # use for loop to generate higher nodes
    for i in range(1,level_number):
        nodes_num=num_bottom_level_node//np.prod(tree_num_child[:i])
        tmpa,tmpb=initialize_tree(i, nodes_num,given_distribution[i],mustretire[i],factionmustretire[i])
        data=np.append(data,tmpa,axis=0)
        data=np.append(data,tmpb,axis=0)
    
    if topleader!=data[-2,7]:
        data[-2,7],data[-1,7]=data[-1,7],data[-2,7]
    return data



####################################
# Pop retiring person dynamically
####################################
# a function used to count how many politicians at each level for each faction
@jit(nopython=True)
def howmanycofactions_bylevel(data):
    faction_num_bylevel=dict()
    for factionname in np.arange(num_faction):
        # factionname is the number of faction
        # num_bylevel is a list containing numbers of politicians for certain faction
        num_bylevel=np.array([0]*level_number)
        t=0
        for level in np.arange(level_number):
            tocount=data[data[:,4]==level]
            tmpcofaction=tocount[tocount[:,7]==factionname]
            num_bylevel[t]=tmpcofaction.shape[0]
            t+=1
        faction_num_bylevel[factionname]=num_bylevel
    return faction_num_bylevel


@jit(nopython=True)
def candidate_c_score(data,promote_level,partner_faction,cohesion,v_by_level,delta_by_level,w_by_level,w2,w_sum,alpha,tmpseed,age):
    np.random.seed(tmpseed)
    promote_level=int(promote_level)*1
    retired_level=promote_level+1
    # candidates list
    candidate_list=data[data[:,4]==promote_level]*1
    # leader faction
    leader_faction=int(find_faction(data,level_number-1,0,0)+0)
    # faction fraction by level map
    wnum_by_level=howmanycofactions_bylevel(data)

    score=np.array([0.])
    score1=np.array([0.])
    score2=np.array([0.])
    for i in np.arange(candidate_list.shape[0]):
        bornyear=candidate_list[i][0]*1
        minbornyear=np.min(candidate_list[:,0])*1
        endlevel=candidate_list[i][4]*1
        startlevel=candidate_list[i][1]*1
        if endlevel>startlevel:
            score=np.append(score,-1e20)   #  df[i][8]==>ability
            continue             

        # candidate faction
        candidate_faction=int(candidate_list[i][7])  #  df[i][6]==>faction
        # faction cohesion/ faction fraction in upper levels
        theta=cohesion[candidate_faction]
        # co-factional politicians in each level
        # remember to reduce 1 for the level where the candidate being considered also counts 1
        wnum_by_level_specific=wnum_by_level[candidate_faction]*1
        wnum_by_level_specific[promote_level]-=1
        
        # first component of c-score
        v=v_by_level[retired_level]
        if candidate_faction==partner_faction:
            c1=2*v/(3+theta)-2*v/3
        else:
            c1=0.
        
        # second component of c-score
        delta=delta_by_level[retired_level]
        if int(candidate_faction)==int(leader_faction) and leader_faction<num_faction-1: 
            c2=np.sum(wnum_by_level_specific*w_by_level*delta*theta)+w2*theta*delta
#            print(c2)
        else:
            c2=np.sum(wnum_by_level_specific*w_by_level*delta*theta)
        c2=100*c2/w_sum
        
        score=np.append(score,c1+c2+candidate_list[i][8]+ np.random.gumbel(0,1)*alpha)   #  df[i][8]==>ability
    score=score[1:].reshape(-1,1)
    candidate_list=np.append(candidate_list,score,axis=1)

    return candidate_list



@jit(nopython=True)
def politicianlist(data):
    politicians=np.array([[-1.,-1.,-1.]])
    for k in np.arange(data.shape[0]):
        level=data[k,4]
        lr=data[k,5]
        node=data[k,6]
        tmp=np.array([[level,lr,node]])
        politicians=np.append(politicians,tmp,axis=0)
    politicians=politicians[1:]
    return politicians

@jit(nopython=True)
def sortlevellr(df):
    tmp=np.array([0.]*df.shape[0])
    for row in np.arange(df.shape[0]):
        tmp[row]=-df[row][4]*1e5+df[row][5]
    orders=np.argsort(tmp)
    return orders
   

@jit(nopython=True)
def retirelist(data,current_age,tmpseed):
    np.random.seed(tmpseed)
    #random.seed(s)
    # force some politicians to retire
    retirecandidates=data*1
    mustretire=retirecandidates[retirecandidates[:,0]<=current_age-2]*1
#    mustretire=mustretire[mustretire[:,4]>1]*1

    rest_retire=retirecandidates[retirecandidates[:,0]>current_age-2]*1
#    rest_retire2=retirecandidates[retirecandidates[:,4]<=1]*1
#    rest_retire=np.append(rest_retire1,rest_retire2,axis=0)
    
    # exclude top leader from randomly retirement process
    toplevel=np.max(data[:,4])
    rest_retire=rest_retire[rest_retire[:,4]<toplevel]
    
    total=retirecandidates.shape[0]
    num=int(retirecandidates.shape[0]/2)-mustretire.shape[0]
    retire_index=np.array([-1])
    while len(retire_index)<=num:
        number_random=np.random.randint(rest_retire.shape[0])
        if np.any(retire_index==number_random):
            continue
        else:
            retire_index=np.append(retire_index,number_random)  
    retire_index=retire_index[1:]    

    retire_politician_list=mustretire*1
    for i in np.arange(max(num,0)):
        index=retire_index[i]
        tmp=rest_retire[index:index+1]
        retire_politician_list=np.append(retire_politician_list,tmp,axis=0)
    
    orderofretire=sortlevellr(retire_politician_list)
    retire_politician_list=retire_politician_list[orderofretire]    

    return retire_politician_list


@jit(nopython=True)
def samearray(a,b,start=4,end=6):
    if len(b)==3:
        for i in np.arange(start,end+1):
            if a[i]!=b[i-4]:
                return False
    else:
        for i in np.arange(start,end+1):
            if a[i]!=b[i]:
                return False       
    return True

@jit(nopython=True)
def stayorpromote(df,retires):
    sorp=df*1
    signal=True
    t=0
    i=0
    j=0
    while i<df.shape[0]:
        while j<retires.shape[0]:
            if samearray(df[i],retires[j],start=4,end=6):                
                signal=False
                break
            else:
                j+=1
        if signal:
            sorp[t]=df[i]*1
            t+=1                
        signal=True
        j=0
        i+=1
    sorp=sorp[:t]*1
    
    # exclude top leader
    toplevel=np.max(df[:,4])
    sorp=sorp[sorp[:,4]<toplevel]
#    sorp=sorp[sorp[:,4]==0]
    return sorp



@jit(nopython=True)
def find_faction(hisarray,level,lr,node):
    faction=hisarray[(hisarray[:,4]==level) & (hisarray[:,5]==lr) & (hisarray[:,6]==node)][0][7]
    return faction

@jit(nopython=True)
def find_ability(hisarray,level,lr,node):
    faction=hisarray[(hisarray[:,4]==level) & (hisarray[:,5]==lr) & (hisarray[:,6]==node)][0][8]
    return faction

@jit(nopython=True)
def find_data_final(sorp,level,lr,node):
    for row in np.arange(sorp.shape[0]):
        if sorp[row][4]==level and sorp[row][5]==lr and sorp[row][6]==node:
            return row
    return -1

@jit(nopython=True)
def find_sorprow_initial(sorp,level,lr,node,faction,ability):
    for row in np.arange(sorp.shape[0]):
        if sorp[row][1]==level and sorp[row][2]==lr and sorp[row][3]==node \
        and sorp[row][7]==faction and sorp[row][8]==ability:
            return row
    return -1


@jit(nopython=True)
def get_promoted(df):
    promoted=np.array([0.])
    for i in np.arange(df.shape[0]):
        if df[i][4]>df[i][1]:
            promoted=np.append(promoted,1.0) 
        else:
            promoted=np.append(promoted,0.) 
    promoted=promoted[1:].reshape(-1,1)
    df=np.append(df,promoted,axis=1)
    return df

@jit(nopython=True)
def get_retired(df):
    retired=np.array([0.])
    for i in np.arange(df.shape[0]):
        if df[i][4]<0:
            retired=np.append(retired,1.0) 
        else:
            retired=np.append(retired,0.) 
    retired=retired[1:].reshape(-1,1)
    df=np.append(df,retired,axis=1)
    return df

@jit(nopython=True)
def PBSCshare(data):
    maxlevel=np.max(data[:,4])
    pbsc=data[data[:,4]>maxlevel-2]
    totalnum=pbsc.shape[0]
    pbsc=pbsc[:,7]
    pbsc_distribution=np.array([0]*num_faction)
    for each in pbsc:
        pbsc_distribution[int(each)]+=1
    pbsc_distribution=pbsc_distribution/totalnum
    return pbsc_distribution


@jit(nopython=True)
def refresh_data(df):
    for row in np.arange(df.shape[0]):
#        df[row][0]=0
        df[row][1]=df[row][4]*1
        df[row][2]=df[row][5]*1
        df[row][3]=df[row][6]*1
    return df

@jit(nopython=True)
def add_pbsc_leader(sorp,pbsc_distribution,leader_faction):
    pbsccolumn=np.array([0.]*sorp.shape[0])
    leadercolumn=np.array([leader_faction]*sorp.shape[0])
    for row in np.arange(sorp.shape[0]):
        pbsccolumn[row]=pbsc_distribution[int(sorp[row][7])]
    pbsccolumn=pbsccolumn.reshape(-1,1)
    leadercolumn=leadercolumn.reshape(-1,1)
    sorp=np.append(sorp,leadercolumn,axis=1)
    sorp=np.append(sorp,pbsccolumn,axis=1)
    return sorp

@jit(nopython=True)
def same_as_leader(data):
    same=np.array([0.]*data.shape[0])
    for row in np.arange(data.shape[0]):
        if data[row][7]==data[row][9] and data[row][9]!=num_faction-1:
            same[row]=1.0
    same=same.reshape(-1,1)
    data=np.append(data,same,axis=1)
    return data

@jit(nopython=True)
def sort_level_lr_node(df):
    tmp=np.array([0.]*df.shape[0])
    for row in np.arange(df.shape[0]):
        tmp[row]=-df[row][4]*1e10+df[row][5]*1e5+df[row][6]
    orders=np.argsort(tmp)
    return orders   

@jit(nopython=True)
def get_reg2data(data,age):
    data=data[data[:,4]>0]
    firsthand=data[data[:,5]==0]
    secondhand=data[data[:,5]==1]
    
    firsthand=firsthand[sort_level_lr_node(firsthand)]
    secondhand=secondhand[sort_level_lr_node(secondhand)]
    level=firsthand[:,4:5]
    politician1=firsthand[:,7:8]
    politician2=secondhand[:,7:8]
    year=np.ones((politician1.shape[0],1))*age
    
    level=np.append(level,politician1,axis=1)
    level=np.append(level,politician2,axis=1)
    level=np.append(level,year,axis=1)
    return level


"""
parameters: a numpy array consisting of 3 numbers
v - personal utility
lam - leader premia
delta - promotion utility
theta0/1/2/3/4 - faction cohesion for CYLC/SH/P/M/Neutral
"""
@jit(nopython=True)
def retireloop(x0,retires,sorp,data,waitings,age,tmpseed):  
    np.random.seed(tmpseed)
    #######################
    #  Some prework: params
    #######################
    parameters=x0[:8]*1
    cohesion_tmp=x0[8:]*1
    cohesion=np.append(cohesion_tmp,np.array([0]),axis=0) 
    
    # parameters used
    v0,v1,delta0,delta1,w0,w1,w2,alpha=parameters

    # support weight by level
    w_by_level=np.array([w0,w0,w0,w0+w1,w0+w1])
    w_sum=np.sum(politician_number_by_level[::-1]*w_by_level)+w2

    # delta/v by level
    v_by_level=np.array([v0,v0,v0+v1*1,v0+v1*2,v0+v1*2])
    delta_by_level=np.array([delta0,delta0+delta1*1,delta0+delta1*2,delta0+delta1*3,delta0+delta1*3])
    
    while len(retires)>0:

        retire_level,retire_lr,retire_node,retire_faction,retire_ability=retires[0][4:9]*1
        # if a person retires, his promoted=0 even if he is promoted between 2 congress
        rowinsorp=find_sorprow_initial(sorp,retire_level,retire_lr,retire_node, \
                                       retire_faction,retire_ability)*1
        if rowinsorp>=0:
            sorp[rowinsorp][4]=-1

        rowindata=find_data_final(data,retire_level,retire_lr,retire_node)*1
        retire_level,retire_lr,retire_node,retire_faction,retire_ability=data[rowindata][4:9]*1
        retires=retires[1:]*1
        
        newborn_faction=int(retire_faction+0)
        
        while retire_level>0:
            # partner's tree and faction
            if retire_lr==0:
                partner_lr=1
            else:
                partner_lr=0
            partner_faction=int(find_faction(data,retire_level,partner_lr,retire_node)+0)   
            
            # candidate number
            promote_level=retire_level-1
            # calculate each candidate's score, adding it to column 8 
            # also, notice that this column is not added to data!! (data.shape[0] is still 7)
            tmpseed=np.random.randint(0,2**31)
            candidate_list=candidate_c_score(data,promote_level,partner_faction,cohesion,
                                             v_by_level,delta_by_level,w_by_level,w2,w_sum,alpha,tmpseed,age)
                        
            maxscore=np.max(candidate_list[:,9])  # columns 8 ==> score of being promoted
            hisarray=candidate_list[candidate_list[:,9]==maxscore][0]
            promoted_level_initial=hisarray[1]*1
            promoted_lr_initial=hisarray[2]*1
            promoted_node_initial=hisarray[3]*1            
            promoted_level=hisarray[4]*1
            promoted_lr=hisarray[5]*1
            promoted_node=hisarray[6]*1
            promoted_faction=hisarray[7]*1
            promoted_ability=hisarray[8]*1
            
            # update the person's status in sorp: only update the final level/lr/node !!!
            rowinsorp=find_sorprow_initial(sorp,promoted_level_initial,promoted_lr_initial, \
                            promoted_node_initial,promoted_faction,promoted_ability)*1
            if rowinsorp>=0:
#                sorp[rowinsorp][0]=age*1
                sorp[rowinsorp][4]=retire_level*1
                sorp[rowinsorp][5]=retire_lr*1
                sorp[rowinsorp][6]=retire_node*1
        
            #  use promoted person's data to cover retired person's
            retired_row=find_data_final(data,retire_level,retire_lr,retire_node)*1
            data[retired_row][0]=age*1
            data[retired_row][1]=hisarray[1]*1
            data[retired_row][2]=hisarray[2]*1
            data[retired_row][3]=hisarray[3]*1
            data[retired_row][7]=promoted_faction*1
            data[retired_row][8]=promoted_ability*1
            
            
            # take promoted person's position as retired
            retire_level=promoted_level*1
            retire_lr=promoted_lr*1
            retire_node=promoted_node*1
            
        
        # for bottom politician, we simply replace his ability and faction
        if retire_level==0:                
            # partner's tree and faction
            if retire_lr==0:
                partner_lr=1
            else:
                partner_lr=0
            partner_faction=int(find_faction(data,retire_level,partner_lr,retire_node)+0) 
            
            tmpdata=np.append(data,waitings,axis=0)*1
            candidate_list=candidate_c_score(tmpdata,-1,partner_faction,cohesion,
                                             v_by_level,delta_by_level,w_by_level,w2,w_sum,alpha,tmpseed,age)

            maxscore=np.max(candidate_list[:,9])  # columns 8 ==> score of being promoted
            hisarray=candidate_list[candidate_list[:,9]==maxscore][0]
            promoted_level_initial=hisarray[1]*1
            promoted_lr_initial=hisarray[2]*1
            promoted_node_initial=hisarray[3]*1            
            promoted_level=hisarray[4]*1
            promoted_lr=hisarray[5]*1
            promoted_node=hisarray[6]*1
            promoted_faction=hisarray[7]*1
            promoted_ability=hisarray[8]*1

            #  use promoted person's data to cover retired person's
            retired_row=find_data_final(data,retire_level,retire_lr,retire_node)*1
            data[retired_row][0]=age*1
            data[retired_row][1]=promoted_level_initial*1
            data[retired_row][2]=promoted_lr_initial*1
            data[retired_row][3]=promoted_node_initial*1
            data[retired_row][7]=promoted_faction*1
            data[retired_row][8]=promoted_ability*1
            #addsorp=data[rowindata:rowindata+1]*1
            #sorp=np.append(sorp,addsorp,axis=0)
            
            # delete the promoted from waiting list
            rowinwait=find_sorprow_initial(waitings,promoted_level,promoted_lr,promoted_node, \
                                           promoted_faction,promoted_ability)*1  
            waitings[rowinwait,8]=-1e20
            
    return np.append(sorp,data,axis=0)



def get_allsorp2(s,x0,data,waitings):        
    np.random.seed(s)    
    
    # parameters used
    parameters=x0[:8]*1
    v0,v1,delta0,delta1,w0,w1,w2,alpha=parameters
    
    pre_age=0
    datasize0=data.shape[0]

    tmpseed=np.random.randint(0,2**31)
    retires=retirelist(data,1+pre_age,tmpseed)
    sorp=data*1
    # PBSC faction distribution
    tmpseed=np.random.randint(0,2**31)
    
    tmp=retireloop(x0,retires,sorp,data,waitings,1+pre_age,tmpseed)
    tmpsize0=tmp.shape[0]
    stopticker=tmpsize0-datasize0
    # add PBSC and leader_faction into sorp
    sorp=tmp[:stopticker]*1
    data=tmp[stopticker:]*1
    data=refresh_data(data)*1
 
    return [sorp,data]



def faciton_distribution(updatedata):
    distribution=dict()
    for i in range(level_number):
        tmpdata=updatedata[updatedata[:,1]==i]
        
        tmpdict={0:0,1:0,2:0,3:0,4:0}
        leveldistri=Counter(tmpdata[:,7])
        for j in range(num_faction): 
            if j in leveldistri:
                tmpdict[j]=leveldistri[j]
        distribution[i]=tmpdict
    return distribution



def generate_data(given_distribution,mustretire,factionmustretire,x0,share_benchmark, \
                  nbr_benchmark, leaderfaction, path, pathtag, version):
    
    np.random.seed(0)
    random.seed(0)

    # parameters used
    parameters=x0[:8]*1
    v0,v1,delta0,delta1,w0,w1,w2,alpha=parameters
    
    # initiate data    
    data=init_database(given_distribution,mustretire,factionmustretire,topleader=leaderfaction)
    
    # for raw data distribution
    distribution0=faciton_distribution(data)
    raw=[list(i.values()) for i in distribution0.values()]
    raw=pd.DataFrame(raw)
    raw=raw.rename(columns={0:'CYLC',1:'Shanghai',2:'Princelings',3:'Military',4:'Neutral'}, \
               index={0:'AC',1:'CC',2:'PB',3:'SC',4:'Top leader'})
    
    
    # begin to loop, and get simulated data
    total=[]
    promoted=[]
    
    for i in range(1000):
        s=seed_true[i]
        waiting_distribution=[int(i) for i in faction_prob*200][:-1]
        norestrict={0:0,1:0,2:0,3:0,4:0}
        waitings= initialize_tree(-1, 100, waiting_distribution, norestrict, list(norestrict.values()))
        level0_a=waitings[0]
        level0_b=waitings[1]
        # all trees save in a list
        waitings=np.append(level0_a,level0_b,axis=0)
        
        tmpdata=data*1
        sorp,updatedata=get_allsorp2(s,x0,tmpdata,waitings)*1
        
        distribution=faciton_distribution(updatedata)
        res=[list(i.values()) for i in distribution.values()]
        res=pd.DataFrame(res)
        res=res.rename(columns={0:'CYLC',1:'Shanghai',2:'Princelings',3:'Military',4:'Neutral'}, \
                   index={0:'AC',1:'CC',2:'PB',3:'SC',4:'Top leader'})
        total.append(res)
        
        distribution_sorp=faciton_distribution(sorp[sorp[:,4]>sorp[:,1]])
        res=[list(i.values()) for i in distribution_sorp.values()]
        res=pd.DataFrame(res)
        res=res.rename(columns={0:'CYLC',1:'Shanghai',2:'Princelings',3:'Military',4:'Neutral'}, \
                   index={0:'AC',1:'CC',2:'PB',3:'SC',4:'Top leader'})
        promoted.append(res)


    total=pd.concat(total,axis=0)
    total.reset_index(inplace=True)    
    promoted=pd.concat(promoted,axis=0)
    promoted.reset_index(inplace=True) 
    
    asetofdata=[raw,total,promoted,given_distribution,mustretire,factionmustretire,x0,share_benchmark, \
                  nbr_benchmark, leaderfaction, path, pathtag,version]
    with open(path+'data_'+pathtag+'.pkl','wb') as in_data:
        pickle.dump(asetofdata,in_data)
    return



def plots(pathtag,path): 
    f1 = open(path+'data_'+pathtag+'.pkl', 'rb')
    raw,total,promoted,given_distribution,mustretire,factionmustretire,x0,share_benchmark, \
                  nbr_benchmark, leaderfaction, path, pathtag,version= pickle.load(f1)
    f1.close()
    f1 = open(path+'data_'+'Neutral'+'.pkl', 'rb')
    rawN,totalN,promotedN,given_distributionN,mustretireN,factionmustretireN,x0N,share_benchmarkN, \
                  nbr_benchmarkN, leaderfactionN, pathN, pathtagN,versionN= pickle.load(f1)
    f1.close()
           
    totalmeanN=totalN.groupby('index').mean()
    totalmeanN['sum']=totalmeanN.sum(axis=1)                               
                  
    totalmean=total.groupby('index').mean()
    totalmean['sum']=totalmean.sum(axis=1)
    totalerror=abs(totalmean-nbr_benchmark)
    totalerror['RMSE']=(totalerror*totalerror).sum(axis=1)**0.5
    totalerror.loc['RMSE']=(totalerror*totalerror).sum(axis=0)**0.5

    totalmean2=copy.deepcopy(totalmean)
    totalmean2['Neutral']=totalmean2['Neutral']+totalmean2['Military']+totalmean2['Princelings']
    nbr_benchmark2=copy.deepcopy(nbr_benchmark)
    nbr_benchmark2['Neutral']=nbr_benchmark2['Neutral']+nbr_benchmark2['Military']+nbr_benchmark2['Princelings']
    totalmean2=totalmean2[['CYLC','Shanghai','Neutral']]
    nbr_benchmark2=nbr_benchmark2[['CYLC','Shanghai','Neutral']]
    totalerror2=abs(totalmean2-nbr_benchmark2)
    totalerror2['RMSE']=(totalerror2*totalerror2).sum(axis=1)**0.5
    totalerror2.loc['RMSE']=(totalerror2*totalerror2).sum(axis=0)**0.5

    
    promotedmean=promoted.groupby('index').mean()
    promotederror=abs(promotedmean-distpromoted)
    promotederror['RMSE']=(promotederror*promotederror).sum(axis=1)**0.5
    promotederror.loc['RMSE']=(promotederror*promotederror).sum(axis=0)**0.5
    
    promotedmean2=copy.deepcopy(promotedmean)
    promotedmean2['Neutral']=promotedmean2['Neutral']+promotedmean2['Military']+promotedmean2['Princelings']
    distpromoted2=copy.deepcopy(distpromoted)
    distpromoted2['Neutral']=distpromoted2['Neutral']+distpromoted2['Military']+distpromoted2['Princelings']
    promotedmean2=promotedmean2[['CYLC','Shanghai','Neutral']]
    distpromoted2=distpromoted2[['CYLC','Shanghai','Neutral']]
    promotederror2=abs(promotedmean2-distpromoted2)
    promotederror2['RMSE']=(promotederror2*promotederror2).sum(axis=1)**0.5
    promotederror2.loc['RMSE']=(promotederror2*promotederror2).sum(axis=0)**0.5    
        
    # simulated faction share and output
    simulatedshare=totalmean.apply(lambda x: x/x['sum'],axis=1)[['CYLC','Shanghai','Princelings','Military','Neutral']]
    simulatedshare.reset_index().rename(columns={'index':''}).round(3).to_latex(index=False, \
                          encoding='utf-8',buf=path+"/forecasting_"+pathtag+".tex")

    return None




version="/speed_v5_1up1congress_corrected/"
path= "C:\\Users\\zhuoli\\Desktop\\China_Project\\Results\\Forecasting\\"


distpromoted=pd.DataFrame( \
[ \
['AC', 2, 4, 2, 3, 26],
['CC', 0, 0, 1, 1, 8],
['PB', 0, 2, 0, 0, 3],
['SC', 0, 0, 0, 0, 0],
['Top leader',0,0,0,0,0]],
columns=["","CYLC", "Shanghai", "Princelings","Military","Neutral"]).set_index("")

dist19sh=pd.DataFrame( \
[ \
['AC', 8, 9, 1, 25,129],
['CC', 18, 12, 2, 34,112],
['PB', 1,  3, 1, 2,12],
['SC', 0,  2, 0, 0,3],
['Top leader',1,1,0,0,0]],
columns=["","CYLC", "Shanghai", "Princelings","Military","Neutral"]).set_index("")
tmpdist19sh=dist19sh
tmpdist19sh['sum']=dist19sh.sum(axis=1)
dist19shpct=tmpdist19sh.apply(lambda x: x/x['sum'],axis=1)

outhere2=path+r"\speed_v5_1up1congress_corrected"+"\\real19sh.tex"
outhere4=path+"\\real19shpct.tex"
# dist19sh.reset_index().to_latex(index=False,encoding='utf-8',buf=outhere2)
tmpout=dist19shpct[["CYLC", "Shanghai", "Princelings","Military","Neutral"]]
tmpout.reset_index().round(3).to_latex(index=False,encoding='utf-8',buf=outhere4)

dist19shpct=tmpdist19sh.apply(lambda x: x/x['sum'],axis=1)[["CYLC", "Shanghai", "Princelings","Military"]]



#################################
# Top leader as Shanghai
#################################
given_distribution={0:[11,6,6,18],1:[15,5,8,31],2:[3,2,3,2],3:[1,0,3,0],4:[1,1,0,0]}
mustretire={0:45,1:59,2:3,3:2,4:0}
factionmustretire={0:[2,2,0,2,39],1:[8,1,6,7,37],2:[1,0,1,0,1],3:[0,0,1,0,0],4:[0,0,0,0,0]}
x0=np.array([4.5, 101, 0.85, 0.04, 0.1, 10.07, 10.81, 7.56, 0.82, 0.85, 0.0, 0.038])

generate_data(given_distribution,mustretire,factionmustretire,x0,dist19shpct, \
                  dist19sh, 1, path, 'Shanghai', version)

#################################
# Top leader as All 0
#################################
given_distribution={0:[11,6,6,18],1:[15,5,8,31],2:[3,2,3,2],3:[1,0,3,0],4:[1,0,1,0]}
mustretire={0:45,1:59,2:3,3:2,4:0}
factionmustretire={0:[2,2,0,2,39],1:[8,1,6,7,37],2:[1,0,1,0,1],3:[0,0,1,0,0],4:[0,0,0,0,0]}
x0=np.array([4.5, 101, 0.85, 0.04, 0.1, 10.07, 10.81, 7.56, 0, 0, 0.0, 0])

generate_data(given_distribution,mustretire,factionmustretire,x0,dist19shpct, \
                  dist19sh, 2, path, 'Neutral', version)


plots('Shanghai',path)




