# -*- coding: utf-8 -*-
"""
In this version, we assume the following things:
    
1. For Military and Princelings, we only match the sum of moments
2. For CYLC/SH, we assume the proportion of leader-premia to SC-premia are same
"""

import numpy as np
import pandas as pd
from collections import Counter
import statsmodels.api as sm
from sklearn.linear_model import LinearRegression

from parameters import *
np.seterr(divide='ignore', invalid='ignore')

def call_moments(reg1,reg2):
    ###################
    # clean data
    ###################
    reg3=pd.DataFrame(reg1)
    reg3['Faction']=reg3[7]
    reg4=pd.get_dummies(reg3,columns=[7])
    
    # rename columns
    reg4=reg4.rename(columns={0:'Birth',1:'Initial_level',2:'Initial_lr',3:'Initial_node'
                              ,4:'Final_level',5:'Final_lr',6:'Final_node'
                              ,8:'Ability',9:'Leader',10:'PBSC_share',11:'Promotion',12:'Leadersame'
                              ,'7_0.0':'Faction0','7_1.0':'Faction1','7_2.0':'Faction2','7_3.0':'Faction3'
                              ,'7_4.0':'Faction4'})
    regression1=reg4[['Promotion','Birth','Initial_level','Faction0','Faction1','Faction2','Faction3','Faction', 
                      'Leader','Leadersame','PBSC_share','Ability']]*1
    
    # find which faction has leader in history
    leaderlist=[int(i+0.1) for i in list(Counter(regression1['Leader'].values).keys()) if i!=num_faction-1]
    exog_vars = ['Ability']
    for i in leaderlist:
        s1='Leader_Faction'+str(i)
        s2='Faction'+str(i)
        regression1[s1]=regression1['Leadersame']*regression1[s2]
        exog_vars.append(s1)
    
    # create PBSC premia
    for i in range(num_faction-1):
        s2='Faction'+str(i)
        s3='PBSC_Faction'+str(i)
        regression1[s3]=regression1['PBSC_share']*regression1[s2]
        exog_vars.append(s2)
        exog_vars.append(s3)
    
    # for table 5
    reg5=pd.DataFrame(reg2)
    regression2=reg5.rename(columns={0:'Level',1:'Politician1',2:'Politician2',3:'Year'})
    
    regression1['Initiallevel']=regression1['Initial_level']
    regression1['Birth']=regression1['Birth'].astype('int')
    regression1['Initial_level']=regression1['Initial_level'].astype('int')
    regression1=regression1[['Promotion']+exog_vars+['Birth','Initial_level','Initiallevel','Faction']]
    
    # fixed effect
    birth_fixed=['Birth_{}'.format(int(i)) for i in regression1['Birth'].unique()]
    level_fixed=['Initial_level_{}'.format(int(i)) for i in regression1['Initial_level'].unique()]
    exog_vars2=exog_vars+birth_fixed
    exog_vars2+=level_fixed   
    regression1=pd.get_dummies(regression1,columns=['Birth'])    
    regression1=pd.get_dummies(regression1,columns=['Initial_level'])  
 
    ###################
    # Table 3
    ###################
    # FE xtreg: no need for intercept - same result as Stata
    res = sm.OLS(regression1['Promotion'], regression1[exog_vars2])
    res2 = res.fit()
    t_stats=res2.tvalues
    betas=res2.params
    
    moments=dict()
    for var in exog_vars:
        contents,faction=moments_table[var]
        if contents in moments:
            moments[contents][faction]=betas[var]
        else:
            moments[contents]={}
            moments[contents][faction]=betas[var]
    abilityrsq=np.var(regression1['Ability'])*betas['Ability']**2/np.var(regression1.Promotion)
    moments['Ability R-squared']={'total':abilityrsq}
   
    r2=0
    tmp=[i for i in exog_vars if 'Faction' in i]
    for var in tmp:
        r2+=np.var(regression1[var])*betas[var]**2/np.var(regression1.Promotion)
    moments['Faction R-squared']={'total':r2}
    


    
    ###################
    # Table 2
    ###################
    ac=regression1[regression1['Initiallevel']==0]
    cc=regression1[regression1['Initiallevel']==1]
    
    exog_vars = ['Faction0','Faction1','Faction2','Faction3','Ability']
    exog_vars+=birth_fixed
    
    # AC
    # sometimes, const term is deleted due to collinearity
    exog = ac[exog_vars].assign(Intercept=1)
    res = sm.OLS(ac['Promotion'], exog)
    res2 = res.fit()
    ac_stats=res2.tvalues
    ac_betas=res2.params
    ac0to4=ac_betas[['Faction0','Faction1','Faction2','Faction3']]*1
    predicts=pd.DataFrame(res2.predict(exog)).rename(columns={0:'prediction'})
    ac=pd.concat([ac,predicts],axis=1)
    p_ac=[]
    for i in range(num_faction):
        p_ac.append(ac[ac['Faction']==i]['prediction'].mean())
    p_ac=np.array(p_ac)
    pac=[]
    for i in range(num_faction-1):
        pac.append(ac0to4['Faction'+str(i)]+p_ac[-1])
    pac=np.array(pac)/p_ac[-1]
    
    # CC
    exog = cc[exog_vars].assign(Intercept=1)
    res = sm.OLS(cc['Promotion'], exog)
    res2 = res.fit()
    cc_stats=res2.tvalues
    cc_betas=res2.params
    cc0to4=cc_betas[['Faction0','Faction1','Faction2','Faction3']]*1
    predicts=pd.DataFrame(res2.predict(exog)).rename(columns={0:'prediction'})
    cc=pd.concat([cc,predicts],axis=1)
    p_cc=[]
    for i in range(num_faction):
        p_cc.append(cc[cc['Faction']==i]['prediction'].mean())
    p_cc=np.array(p_cc)
    pcc=[]
    for i in range(num_faction-1):
        pcc.append(cc0to4['Faction'+str(i)]+p_cc[-1])
    pcc=np.array(pcc)/p_cc[-1]
    
    ccacrati0=pcc/pac
    moments['Faction premia: CC/AC ratio']= \
        {'CYLC':ccacrati0[0],'Shanghai':ccacrati0[1],'Princelings':ccacrati0[2],'Military':ccacrati0[3]}
   

    
    ###################
    # Table 5
    ###################
    regression2['Level_copy']=regression2['Level']
#    regression2 = pd.get_dummies(regression2,columns=['Level'])
    
    provincial=regression2[regression2['Level']==1]
    provincial=pd.get_dummies(provincial,columns=['Politician1','Politician2'])
    
    national=regression2[regression2['Level']>1]
    national=pd.get_dummies(national,columns=['Politician1','Politician2'])
    
    
    tmpprovincial={}
    tmpnational={}
    tmpprovincial_t={}
    tmpnational_t={}   
    for i in range(4):
        y='Politician1_'+str(i)+'.0'
        x='Politician2_'+str(i)+'.0'
        
        if x in provincial.columns and y in provincial.columns:
            exog = provincial[[x]].assign(Intercept=1)
            #exog = sm.add_constant(provincial[[x]])
            res = sm.OLS(provincial[[y]], exog)
            res2 = res.fit()
            tstats=res2.tvalues
            betas=res2.params
            const=betas['Intercept']
            tmpprovincial[i]=(betas[x]+const)/const
            tmpprovincial_t[i]=tstats[x]
            
        if x in national.columns and y in national.columns:        
            exog = national[[x]].assign(Intercept=1)
            #exog = sm.add_constant(national[[x]])
            res = sm.OLS(national[[y]], exog)
            res2 = res.fit()
            tstats=res2.tvalues
            betas=res2.params
            const=betas['Intercept']
            tmpnational[i]=(betas[x]+const)/const
            tmpnational_t[i]=tstats[x]
    
    moments['Same-faction discount: national']={}
    moments['Same-faction discount: provincial']={}
    if 0 in tmpnational:
        moments['Same-faction discount: national']['CYLC']=tmpnational[0]
    if 1 in tmpnational:
        moments['Same-faction discount: national']['Shanghai']=tmpnational[1]   
    if 2 in tmpnational:
        moments['Same-faction discount: national']['Princelings']=tmpnational[2] 
    if 3 in tmpnational:
        moments['Same-faction discount: national']['Military']=tmpnational[3] 
         
    if 0 in tmpprovincial:
        moments['Same-faction discount: provincial']['CYLC']=tmpprovincial[0]
    if 1 in tmpprovincial:
        moments['Same-faction discount: provincial']['Shanghai']=tmpprovincial[1]
    if 2 in tmpprovincial:
        moments['Same-faction discount: provincial']['Princelings']=tmpprovincial[2]
    if 3 in tmpprovincial:
        moments['Same-faction discount: provincial']['Military']=tmpprovincial[3]
        
    return moments


def call_moments_MP(regression1,regression2,s,e):
    ###################
    # Table 3
    ###################
    birth_fixed=['Birth_{}.0'.format(i) for i in range(s+1,e+1)]
    level_fixed=['Initial_level_{}.0'.format(i) for i in range(1,4)]
    
    exog_vars = ['Faction0','Leader_Faction0','PBSC_Faction0',
                 'Faction1','Leader_Faction1','PBSC_Faction1',
                 'Faction2','Leader_Faction2','PBSC_Faction2',
                 'Faction3','Leader_Faction3','PBSC_Faction3',
                 'Ability']
    exog_vars+=birth_fixed
    exog_vars+=level_fixed
    
#    reg1 = LinearRegression().fit(regression1[exog_vars], regression1['Promotion'])
#    re_res=reg1.coef_

    #exog = sm.add_constant(regression1[exog_vars])
    exog = regression1[exog_vars].assign(Intercept=1)
    res = sm.OLS(regression1['Promotion'], exog)
    res2 = res.fit()
    t_stats=res2.tvalues
    betas=res2.params
    
    
    moments=dict()
    moments['Faction premia: leader']={'Princelings':betas['Leader_Faction2'],'Military':betas['Leader_Faction3']}
    moments['Faction premia: SC share']={'Princelings':betas['PBSC_Faction2'],'Military':betas['PBSC_Faction3']}
    return moments



def dict2pd(m):
    res=pd.DataFrame(columns=list(range(3)))
    for keys1 in m.keys():
        tmp=m[keys1]
        for keys2 in tmp.keys():
            tmplist=[keys1,keys2,tmp[keys2]]
            tmppd=pd.DataFrame(tmplist).T
            res=pd.concat([res,tmppd])
    return res














