#!/user/zw2536/.conda/envs/python369/bin/python
"""
1- add life span
2- draw from type I extreme distribution rather than unifrom distribution
"""


import os


import pandas as pd
import random
import numpy as np
from collections import Counter
from sklearn.linear_model import LinearRegression
import time
import multiprocessing
#from multiprocessing import Process
from multiprocessing import freeze_support
from multiprocessing import Pool
from multiprocessing import Queue


import pickle
from numba import jit
import numba as nb

from get_moments import *
from table_moments import *
from optimizer import *
from parameters import *




####################################
# Initialize tree
####################################
def initialize_tree(level,nodes_num,alpha):
    num_bottom_level_node=nodes_num
    
    # 2 dataframe to initialize: a,b are counterparts in the same level-tree
    # eg. node 1 at level0_a and node 1 at level0_b are partners
    level0_a=pd.DataFrame(pd.Series(range(num_bottom_level_node)))
    level0_b=pd.DataFrame(pd.Series(range(num_bottom_level_node)))
    level0_a=level0_a.rename(columns={0:"Node"})
    level0_b=level0_b.rename(columns={0:"Node"})
    level0_a['Node_v2']=level0_a['Node']
    level0_b['Node_v2']=level0_b['Node']  
    
    # left or right
    level0_a['LR']=0
    level0_b['LR']=1
    # final node
    level0_a['LR_v2']=0
    level0_b['LR_v2']=1
    
    # level
    level0_a['Level']=level
    level0_b['Level']=level
    # level
    level0_a['Level_v2']=level
    level0_b['Level_v2']=level
    
    # ability
    level0_a['Ability']=level0_a.apply(lambda x: np.random.uniform(0,10)/alpha, axis=1)
    level0_b['Ability']=level0_a.apply(lambda x: np.random.uniform(0,10)/alpha, axis=1)
    # faction
    level0_a['Faction']=level0_a.apply(lambda x: np.random.choice(num_faction, 1, p=faction_prob)[0], axis=1)
    level0_b['Faction']=level0_a.apply(lambda x: np.random.choice(num_faction, 1, p=faction_prob)[0], axis=1)

    # birth
    level0_a['Birth']=0
    level0_b['Birth']=0
    
    level0_a=level0_a[['Birth','Level','LR','Node','Level_v2','LR_v2','Node_v2','Faction','Ability']]
    level0_b=level0_b[['Birth','Level','LR','Node','Level_v2','LR_v2','Node_v2','Faction','Ability']]
    
    return np.array(level0_a),np.array(level0_b)



def init_database(alpha):    
    tmpdata= initialize_tree(0, num_bottom_level_node,alpha)
    level0_a=tmpdata[0]
    level0_b=tmpdata[1]
    # all trees save in a list
    data=np.append(level0_a,level0_b,axis=0)
    
    # use for loop to generate higher nodes
    for i in range(1,level_number):
        nodes_num=num_bottom_level_node//np.prod(tree_num_child[:i])
        tmpa,tmpb=initialize_tree(i, nodes_num,alpha)
        data=np.append(data,tmpa,axis=0)
        data=np.append(data,tmpb,axis=0)

    return data



####################################
# Pop retiring person dynamically
####################################
# a function used to count how many politicians at each level for each faction
@jit(nopython=True)
def howmanycofactions_bylevel(data):
    faction_num_bylevel=dict()
    for factionname in np.arange(num_faction):
        # factionname is the number of faction
        # num_bylevel is a list containing numbers of politicians for certain faction
        num_bylevel=np.array([0]*level_number)
        t=0
        for level in np.arange(level_number):
            tocount=data[data[:,4]==level]
            tmpcofaction=tocount[tocount[:,7]==factionname]
            num_bylevel[t]=tmpcofaction.shape[0]
            t+=1
        faction_num_bylevel[factionname]=num_bylevel
    return faction_num_bylevel

@jit(nopython=True)
def howmanycofactions_bylevel_v2(data):
    faction_num_bylevel=dict()
    for factionname in np.arange(num_faction):
        # factionname is the number of faction
        # num_bylevel is a list containing numbers of politicians for certain faction
        num_bylevel=np.array([0]*level_number)
        t=0
        for level in np.arange(level_number):
            tocount=data[data[:,4]==level]
            tmpcofaction=tocount[tocount[:,7]==factionname]
            num_bylevel[t]=tmpcofaction.shape[0]
            t+=1
        faction_num_bylevel[factionname]=num_bylevel
    res=faction_num_bylevel[0].reshape(-1,1)*1
    for factionname in np.arange(1,num_faction):
        tmpdata=faction_num_bylevel[factionname].reshape(-1,1)*1
        res=np.append(res,tmpdata,axis=1)*1
    ladder=np.array([0,1,2,3,4]).reshape(-1,1)
    res=np.append(ladder,res,axis=1)*1
    return res


@jit(nopython=True)
def candidate_c_score(data,promote_level,partner_faction,cohesion,v_by_level,delta_by_level,w_by_level,w2,w_sum,alpha,tmpseed,age):
    np.random.seed(tmpseed)
    promote_level=int(promote_level)*1
    retired_level=promote_level+1
    # candidates list
    candidate_list=data[data[:,4]==promote_level]*1
    # leader faction
    leader_faction=int(find_faction(data,level_number-1,0,0)+0)
    # faction fraction by level map
    wnum_by_level=howmanycofactions_bylevel(data)

    score=np.array([0.])
    score1=np.array([0.])
    score2=np.array([0.])
    for i in np.arange(candidate_list.shape[0]):
        bornyear=candidate_list[i][0]*1
        minbornyear=np.min(candidate_list[:,0])*1
        endlevel=candidate_list[i][4]*1
        startlevel=candidate_list[i][1]*1
        if endlevel>startlevel:
            score=np.append(score,-1e20)   #  df[i][8]==>ability
            continue             

        # candidate faction
        candidate_faction=int(candidate_list[i][7])  #  df[i][6]==>faction
        # faction cohesion/ faction fraction in upper levels
        theta=cohesion[candidate_faction]
        # co-factional politicians in each level
        # remember to reduce 1 for the level where the candidate being considered also counts 1
        wnum_by_level_specific=wnum_by_level[candidate_faction]*1
        wnum_by_level_specific[promote_level]-=1
        
        # first component of c-score
        v=v_by_level[retired_level]
        if candidate_faction==partner_faction:
            c1=2*v/(3+theta)-2*v/3
        else:
            c1=0.
        
        # second component of c-score
        delta=delta_by_level[retired_level]
        if int(candidate_faction)==int(leader_faction) and leader_faction<num_faction-1: 
            c2=np.sum(wnum_by_level_specific*w_by_level*delta*theta)+w2*theta*delta
#            print(c2)
        else:
            c2=np.sum(wnum_by_level_specific*w_by_level*delta*theta)
        c2=100*c2/w_sum
        
        score=np.append(score,c1/alpha+c2/alpha+candidate_list[i][8]+ np.random.gumbel(0,1)*alpha/alpha)   #  df[i][8]==>ability
    score=score[1:].reshape(-1,1)
    candidate_list=np.append(candidate_list,score,axis=1)

    return candidate_list



@jit(nopython=True)
def politicianlist(data):
    politicians=np.array([[-1.,-1.,-1.]])
    for k in np.arange(data.shape[0]):
        level=data[k,4]
        lr=data[k,5]
        node=data[k,6]
        tmp=np.array([[level,lr,node]])
        politicians=np.append(politicians,tmp,axis=0)
    politicians=politicians[1:]
    return politicians

@jit(nopython=True)
def sortlevellr(df):
    tmp=np.array([0.]*df.shape[0])
    for row in np.arange(df.shape[0]):
        tmp[row]=-df[row][4]*1e5+df[row][5]
    orders=np.argsort(tmp)
    return orders
   

@jit(nopython=True)
def retirelist(data,current_age,tmpseed):
    np.random.seed(tmpseed)
    #random.seed(s)
    # force some politicians to retire
    retirecandidates=data*1
    mustretire=retirecandidates[retirecandidates[:,0]<=current_age-2]*1
#    mustretire=mustretire[mustretire[:,4]>1]*1

    rest_retire=retirecandidates[retirecandidates[:,0]>current_age-2]*1
#    rest_retire2=retirecandidates[retirecandidates[:,4]<=1]*1
#    rest_retire=np.append(rest_retire1,rest_retire2,axis=0)
    
    # exclude top leader from randomly retirement process
    toplevel=np.max(data[:,4])
    rest_retire=rest_retire[rest_retire[:,4]<toplevel]
    
    total=retirecandidates.shape[0]
    num=int(retirecandidates.shape[0]/2)-mustretire.shape[0]
    retire_index=np.array([-1])
    while len(retire_index)<=num:
        number_random=np.random.randint(rest_retire.shape[0])
        if np.any(retire_index==number_random):
            continue
        else:
            retire_index=np.append(retire_index,number_random)  
    retire_index=retire_index[1:]    

    retire_politician_list=mustretire*1
    for i in np.arange(max(num,0)):
        index=retire_index[i]
        tmp=rest_retire[index:index+1]
        retire_politician_list=np.append(retire_politician_list,tmp,axis=0)
    
    orderofretire=sortlevellr(retire_politician_list)
    retire_politician_list=retire_politician_list[orderofretire]    

    return retire_politician_list


@jit(nopython=True)
def samearray(a,b,start=4,end=6):
    if len(b)==3:
        for i in np.arange(start,end+1):
            if a[i]!=b[i-4]:
                return False
    else:
        for i in np.arange(start,end+1):
            if a[i]!=b[i]:
                return False       
    return True

@jit(nopython=True)
def stayorpromote(df,retires):
    sorp=df*1
    signal=True
    t=0
    i=0
    j=0
    while i<df.shape[0]:
        while j<retires.shape[0]:
            if samearray(df[i],retires[j],start=4,end=6):                
                signal=False
                break
            else:
                j+=1
        if signal:
            sorp[t]=df[i]*1
            t+=1                
        signal=True
        j=0
        i+=1
    sorp=sorp[:t]*1
    
    # exclude top leader
    toplevel=np.max(df[:,4])
    sorp=sorp[sorp[:,4]<toplevel]
#    sorp=sorp[sorp[:,4]==0]
    return sorp



@jit(nopython=True)
def find_faction(hisarray,level,lr,node):
    faction=hisarray[(hisarray[:,4]==level) & (hisarray[:,5]==lr) & (hisarray[:,6]==node)][0][7]
    return faction

@jit(nopython=True)
def find_ability(hisarray,level,lr,node):
    faction=hisarray[(hisarray[:,4]==level) & (hisarray[:,5]==lr) & (hisarray[:,6]==node)][0][8]
    return faction

@jit(nopython=True)
def find_data_final(sorp,level,lr,node):
    for row in np.arange(sorp.shape[0]):
        if sorp[row][4]==level and sorp[row][5]==lr and sorp[row][6]==node:
            return row
    return -1

@jit(nopython=True)
def find_sorprow_initial(sorp,level,lr,node,faction,ability):
    for row in np.arange(sorp.shape[0]):
        if sorp[row][1]==level and sorp[row][2]==lr and sorp[row][3]==node \
        and sorp[row][7]==faction and sorp[row][8]==ability:
            return row
    return -1


@jit(nopython=True)
def get_promoted(df):
    promoted=np.array([0.])
    for i in np.arange(df.shape[0]):
        if df[i][4]>df[i][1]:
            promoted=np.append(promoted,1.0) 
        else:
            promoted=np.append(promoted,0.) 
    promoted=promoted[1:].reshape(-1,1)
    df=np.append(df,promoted,axis=1)
    return df

@jit(nopython=True)
def get_retired(df):
    retired=np.array([0.])
    for i in np.arange(df.shape[0]):
        if df[i][4]<0:
            retired=np.append(retired,1.0) 
        else:
            retired=np.append(retired,0.) 
    retired=retired[1:].reshape(-1,1)
    df=np.append(df,retired,axis=1)
    return df

@jit(nopython=True)
def PBSCshare(data):
    maxlevel=np.max(data[:,4])
    pbsc=data[data[:,4]>maxlevel-2]
    totalnum=pbsc.shape[0]
    pbsc=pbsc[:,7]
    pbsc_distribution=np.array([0]*num_faction)
    for each in pbsc:
        pbsc_distribution[int(each)]+=1
    pbsc_distribution=pbsc_distribution/totalnum
    return pbsc_distribution


@jit(nopython=True)
def refresh_data(df):
    for row in np.arange(df.shape[0]):
#        df[row][0]=0
        df[row][1]=df[row][4]*1
        df[row][2]=df[row][5]*1
        df[row][3]=df[row][6]*1
    return df

@jit(nopython=True)
def add_pbsc_leader(sorp,pbsc_distribution,leader_faction):
    pbsccolumn=np.array([0.]*sorp.shape[0])
    leadercolumn=np.array([leader_faction]*sorp.shape[0])
    for row in np.arange(sorp.shape[0]):
        pbsccolumn[row]=pbsc_distribution[int(sorp[row][7])]
    pbsccolumn=pbsccolumn.reshape(-1,1)
    leadercolumn=leadercolumn.reshape(-1,1)
    sorp=np.append(sorp,leadercolumn,axis=1)
    sorp=np.append(sorp,pbsccolumn,axis=1)
    return sorp

@jit(nopython=True)
def same_as_leader(data):
    same=np.array([0.]*data.shape[0])
    for row in np.arange(data.shape[0]):
        if data[row][7]==data[row][9] and data[row][9]!=num_faction-1:
            same[row]=1.0
    same=same.reshape(-1,1)
    data=np.append(data,same,axis=1)
    return data

@jit(nopython=True)
def sort_level_lr_node(df):
    tmp=np.array([0.]*df.shape[0])
    for row in np.arange(df.shape[0]):
        tmp[row]=-df[row][4]*1e10+df[row][5]*1e5+df[row][6]
    orders=np.argsort(tmp)
    return orders   

@jit(nopython=True)
def get_reg2data(data,age):
    data=data[data[:,4]>0]
    firsthand=data[data[:,5]==0]
    secondhand=data[data[:,5]==1]
    
    firsthand=firsthand[sort_level_lr_node(firsthand)]
    secondhand=secondhand[sort_level_lr_node(secondhand)]
    level=firsthand[:,4:5]
    politician1=firsthand[:,7:8]
    politician2=secondhand[:,7:8]
    year=np.ones((politician1.shape[0],1))*age
    
    level=np.append(level,politician1,axis=1)
    level=np.append(level,politician2,axis=1)
    level=np.append(level,year,axis=1)
    return level


"""
parameters: a numpy array consisting of 3 numbers
v - personal utility
lam - leader premia
delta - promotion utility
theta0/1/2/3/4 - faction cohesion for CYLC/SH/P/M/Neutral
"""
@jit(nopython=True)
def retireloop(x0,retires,sorp,data,waitings,age,tmpseed):  
    np.random.seed(tmpseed)
    #######################
    #  Some prework: params
    #######################
    parameters=x0[:8]*1
    cohesion_tmp=x0[8:]*1
    cohesion=np.append(cohesion_tmp,np.array([0]),axis=0) 
    
    # parameters used
    v0,v1,delta0,delta1,w0,w1,w2,alpha=parameters

    # support weight by level
    w_by_level=np.array([w0,w0,w0,w0+w1,w0+w1])
    w_sum=np.sum(politician_number_by_level[::-1]*w_by_level)+w2

    # delta/v by level
    v_by_level=np.array([v0,v0,v0+v1*1,v0+v1*2,v0+v1*2])
    delta_by_level=np.array([delta0,delta0+delta1*1,delta0+delta1*2,delta0+delta1*3,delta0+delta1*3])
    
    while len(retires)>0:

        retire_level,retire_lr,retire_node,retire_faction,retire_ability=retires[0][4:9]*1
        # if a person retires, his promoted=0 even if he is promoted between 2 congress
        rowinsorp=find_sorprow_initial(sorp,retire_level,retire_lr,retire_node, \
                                       retire_faction,retire_ability)*1
        if rowinsorp>=0:
            sorp[rowinsorp][4]=-1

        rowindata=find_data_final(data,retire_level,retire_lr,retire_node)*1
        retire_level,retire_lr,retire_node,retire_faction,retire_ability=data[rowindata][4:9]*1
        retires=retires[1:]*1
        
        newborn_faction=int(retire_faction+0)
        
        while retire_level>0:
            # partner's tree and faction
            if retire_lr==0:
                partner_lr=1
            else:
                partner_lr=0
            partner_faction=int(find_faction(data,retire_level,partner_lr,retire_node)+0)   
            
            # candidate number
            promote_level=retire_level-1
            # calculate each candidate's score, adding it to column 8 
            # also, notice that this column is not added to data!! (data.shape[0] is still 7)
            tmpseed=np.random.randint(0,2**31)
            candidate_list=candidate_c_score(data,promote_level,partner_faction,cohesion,
                                             v_by_level,delta_by_level,w_by_level,w2,w_sum,alpha,tmpseed,age)
                        
            maxscore=np.max(candidate_list[:,9])  # columns 8 ==> score of being promoted
            hisarray=candidate_list[candidate_list[:,9]==maxscore][0]
            promoted_level_initial=hisarray[1]*1
            promoted_lr_initial=hisarray[2]*1
            promoted_node_initial=hisarray[3]*1            
            promoted_level=hisarray[4]*1
            promoted_lr=hisarray[5]*1
            promoted_node=hisarray[6]*1
            promoted_faction=hisarray[7]*1
            promoted_ability=hisarray[8]*1
            
            # update the person's status in sorp: only update the final level/lr/node !!!
            rowinsorp=find_sorprow_initial(sorp,promoted_level_initial,promoted_lr_initial, \
                            promoted_node_initial,promoted_faction,promoted_ability)*1
            if rowinsorp>=0:
#                sorp[rowinsorp][0]=age*1
                sorp[rowinsorp][4]=retire_level*1
                sorp[rowinsorp][5]=retire_lr*1
                sorp[rowinsorp][6]=retire_node*1
        
            #  use promoted person's data to cover retired person's
            retired_row=find_data_final(data,retire_level,retire_lr,retire_node)*1
            data[retired_row][0]=age*1
            data[retired_row][1]=hisarray[1]*1
            data[retired_row][2]=hisarray[2]*1
            data[retired_row][3]=hisarray[3]*1
            data[retired_row][7]=promoted_faction*1
            data[retired_row][8]=promoted_ability*1
            
            
            # take promoted person's position as retired
            retire_level=promoted_level*1
            retire_lr=promoted_lr*1
            retire_node=promoted_node*1
            
        
        # for bottom politician, we simply replace his ability and faction
        if retire_level==0:                
            # partner's tree and faction
            if retire_lr==0:
                partner_lr=1
            else:
                partner_lr=0
            partner_faction=int(find_faction(data,retire_level,partner_lr,retire_node)+0) 
            
            tmpdata=np.append(data,waitings,axis=0)*1
            candidate_list=candidate_c_score(tmpdata,-1,partner_faction,cohesion,
                                             v_by_level,delta_by_level,w_by_level,w2,w_sum,alpha,tmpseed,age)

            maxscore=np.max(candidate_list[:,9])  # columns 8 ==> score of being promoted
            hisarray=candidate_list[candidate_list[:,9]==maxscore][0]
            promoted_level_initial=hisarray[1]*1
            promoted_lr_initial=hisarray[2]*1
            promoted_node_initial=hisarray[3]*1            
            promoted_level=hisarray[4]*1
            promoted_lr=hisarray[5]*1
            promoted_node=hisarray[6]*1
            promoted_faction=hisarray[7]*1
            promoted_ability=hisarray[8]*1

            #  use promoted person's data to cover retired person's
            retired_row=find_data_final(data,retire_level,retire_lr,retire_node)*1
            data[retired_row][0]=age*1
            data[retired_row][1]=promoted_level_initial*1
            data[retired_row][2]=promoted_lr_initial*1
            data[retired_row][3]=promoted_node_initial*1
            data[retired_row][7]=promoted_faction*1
            data[retired_row][8]=promoted_ability*1
            #addsorp=data[rowindata:rowindata+1]*1
            #sorp=np.append(sorp,addsorp,axis=0)
            
            # delete the promoted from waiting list
            rowinwait=find_sorprow_initial(waitings,promoted_level,promoted_lr,promoted_node, \
                                           promoted_faction,promoted_ability)*1  
            waitings[rowinwait,8]=-1e20
            
    return np.append(sorp,data,axis=0)




def get_allsorp2(s,x0,data,tag_running="Baseline"):       
    np.random.seed(s)    

    # parameters used
    parameters=x0[:8]*1
    cohesion_tmp=x0[8:]*1
    cohesion=np.append(cohesion_tmp,np.array([0]),axis=0) 
    
    # parameters used
    v0,v1,delta0,delta1,w0,w1,w2,alpha=parameters

    # support weight by level
    w_by_level=np.array([w0,w0,w0,w0+w1,w0+w1])
    w_sum=np.sum(politician_number_by_level*w_by_level)+w2

    # delta/v by level
    v_by_level=np.array([v0,v0,v0+v1*1,v0+v1*2,v0+v1*2])
    delta_by_level=np.array([delta0,delta0+delta1*1,delta0+delta1*2,delta0+delta1*3,delta0+delta1*3])
    
    pre_age=5
    if len(data)==0:
        data=init_database(alpha)
        datasize0=data.shape[0]
        for i in range(pre_age):
            tmpseed=np.random.randint(0,2**31)
            retires=retirelist(data,i,tmpseed)
            sorp=data*1
            # PBSC faction distribution
            pbsc_distribution=PBSCshare(data)
            tmpseed=np.random.randint(0,2**31)
            
            tmpdata= initialize_tree(-1, 100, alpha)
            level0_a=tmpdata[0]
            level0_b=tmpdata[1]
            # all trees save in a list
            tmpdata=np.append(level0_a,level0_b,axis=0)*1
    
            tmp=retireloop(x0,retires,sorp,data,tmpdata,i,tmpseed)
            tmpsize0=tmp.shape[0]
            stopticker=tmpsize0-datasize0
            # top leader in the just passed congress
            leader_faction=find_faction(data,4,0,0)
            # add PBSC and leader_faction into sorp
            sorp=tmp[:stopticker]*1
            data=tmp[stopticker:]*1
            sorp=add_pbsc_leader(sorp,pbsc_distribution,leader_faction)*1        
            data=refresh_data(data)  
    else:
        maxage=np.max(data[:,0])
        diffage=maxage-(pre_age-1)
        data[:,0]-=diffage
        
    datasize0=data.shape[0]    
    reg1=None
    reg2=None
    tmpdistri=None
    for i in range(20):
        tmpseed=np.random.randint(0,2**31)
        retires=retirelist(data,i+pre_age,tmpseed)
        sorp=data*1
        # PBSC faction distribution
        pbsc_distribution=PBSCshare(data)
        tmpseed=np.random.randint(0,2**31)
            
        tmpdata= initialize_tree(-1, 100, alpha)
        level0_a=tmpdata[0]
        level0_b=tmpdata[1]
        # all trees save in a list
        tmpdata=np.append(level0_a,level0_b,axis=0)*1

        tmp=retireloop(x0,retires,sorp,data,tmpdata,i+pre_age,tmpseed)
        tmpsize0=tmp.shape[0]
        stopticker=tmpsize0-datasize0
        # add PBSC and leader_faction into sorp
        sorp=tmp[:stopticker]*1
        data=tmp[stopticker:]*1
        # top leader in the just passed congress
        leader_faction=find_faction(data,4,0,0)
        sorp=add_pbsc_leader(sorp,pbsc_distribution,leader_faction)*1
        sorp=get_promoted(sorp)
        sorp=same_as_leader(sorp)


        snapshot=get_reg2data(data,i+pre_age)*1
        if reg1 is not None:
            reg1=np.append(reg1,sorp,axis=0)
        else:
            reg1=sorp
            
        if reg2 is not None:
            reg2=np.append(reg2,snapshot,axis=0)
        else:
            reg2=snapshot
        data=refresh_data(data)
        
        if tmpdistri is not None:
            tmp=howmanycofactions_bylevel_v2(data[data[:,4]>=0])*1
            tmpdistri=np.append(tmpdistri,tmp,axis=0)
        else:
            tmpdistri=howmanycofactions_bylevel_v2(data[data[:,4]>=0])
 
    return [data,tmpdistri]


def get_allsorp1(x0,s,savequeue):
    seed=s[0]
    data,distri=get_allsorp2(seed,x0,[])
    for ss in s[1:]:
        #print(ss)
        data,tmpdistri=get_allsorp2(ss,x0,data)
        distri=np.append(distri,tmpdistri,axis=0)
    savequeue.put(distri)
    return



def print_moments(x0,seeds,allseeds,cores):
    step=int(allseeds/cores)
    savequeue=[multiprocessing.Manager().Queue() for i in range(cores)]
    p = Pool(cores)
    for i in range(cores):
        #print(i)
        seed_totry=np.array(seeds[i*step:(i+1)*step])
        p.apply_async(get_allsorp1, args=(x0,seed_totry,savequeue[i],))
    p.close()
    p.join()

    tmpstore=[]
    for k in savequeue:
        while not k.empty():
            m=k.get()
            tmpstore.append(m)
    distri=tmpstore[0]
    for i in range(1,len(tmpstore)):
        distri=np.append(distri,tmpstore[i],axis=0)
    distri=pd.DataFrame(distri)
    meandistri=distri.rename(columns={0:'ladder'}).groupby(['ladder']).mean(). \
        rename(columns={1:'CYLC',2:'Shanghai',3:'Princelings',4:'Military',5:'Neutral'}, \
               index={0:'AC',1:'CC',2:'PB',3:'SC',4:'Top leader'})
    print(meandistri.to_string())






if __name__=='__main__':

    freeze_support()

    baseline=[4.5, 101, 0.85, 0.04, 0.1, 10.07, 10.81, 7.56, 0.82, 0.85, 0.0, 0.038]
    baseline=np.array(baseline)
    

    all0=[4.5, 101, 0.85, 0.04, 0.1, 10.07, 10.81, 7.56, 0, 0, 0.0, 0]
    all0=np.array(all0)      
   
    print("*****baseline******")
    print_moments(baseline,seed_estimate,2000,10)
    print("*****all0******")
    print_moments(all0,seed_estimate,2000,10)





